package se.ericsson.eto.norarc.javaframe;
import java.io.Serializable;

/**
 * The message queue concept
 * This concept is not intended for JavaFrame users, but only for JavaFrame
 * @see Message
 */
class MailBox implements Serializable {
  private Message firstMessage;
  private Message lastMessage;
  // Invariant: if MailBox is empty, then firstMessage and lastMessage == null;
  // Invariant: lastMessage.nextMessage == null

  protected MailBox() {
  // Creates a new MailBox
    firstMessage = null;
    lastMessage  = null;
  }

  /**
   * Adds the Message sig to the end of this MailBox.
   * @param sig assumed to be != null.
   */
  synchronized
  protected final void addMessage(Message sig) {
    if (firstMessage == null)
      firstMessage = sig;
    else
      lastMessage.nextMessage = sig;
    lastMessage = sig;
    sig.nextMessage = null;
  }

  /**
   * Adds the Message sig to the front of this MailBox.
   * @param sig assumed to be != null.
   */
  synchronized
  protected final void addMessageFirst(Message sig) {
    if (firstMessage == null)
      lastMessage = sig;
    sig.nextMessage = firstMessage;
    firstMessage = sig;
  }

  /**
   * Moves all the messages in mBox to the front of this mailBox.
   * mBox is cleared.
   * @param mBox assumed to be != null.
   */
  synchronized
  protected final void moveMailBox(MailBox mBox) {
    if (mBox.firstMessage == null)
      return;
    if (firstMessage == null)
      lastMessage = mBox.lastMessage;
    else
      mBox.lastMessage.nextMessage = firstMessage;
    firstMessage = mBox.firstMessage;
    mBox.firstMessage = null;
    mBox.lastMessage = null;
  }

  /**
   * Removes the first message of this Mailbox and returns it.
   * If this mailBox is empty, null is returned.
   */
  synchronized
  protected final Message removeFirst() {
    if (firstMessage == null)
      return null;
    Message first = firstMessage;
    if (firstMessage == lastMessage) {
      firstMessage = null;
      lastMessage  = null;
    } else
      firstMessage = firstMessage.nextMessage;
    return first;
  }

  /**
   * Gets without removing the first message of this Mailbox and returns it.
   * If this mailBox is empty, null is returned.
   */
  synchronized
  protected final Message getFirst() {
    return firstMessage;
  }

  /**
   * Removes the Message sig from this Mailbox.
   * If sig is not in this mailBox, the call has no effect.
   */
  synchronized
  protected final void remove(Message sig) {
    Message s = firstMessage;
    if (s == null)
      return;
    if (s == sig) {
      firstMessage = sig.nextMessage;
      if (s == lastMessage)
        lastMessage = null;
      return;
    }
    while (s.nextMessage != null) {
      if (s.nextMessage == sig) {
        s.nextMessage = sig.nextMessage;
        if (sig == lastMessage)
          lastMessage = s;
        return;
      } else {
        s = s.nextMessage;
      }
    }
  }
}
