package se.ericsson.eto.norarc.javaframe;
import java.io.Serializable;

/**
 * The superclass of those objects that represent the concrete message interfaces
 * between {@link ActiveObject}s
 */
public class Mediator implements Addressable, Serializable {
  public Addressable address = null;

  public Mediator() {
  }

  /**
   * Sets the address of this Mediator to the given value.
   * @param newAddress  refers a StateMachine or another Mediator.
   */
  public void addAddress(Addressable newAddress)  throws IllegalArgumentException {
    address = newAddress;
  }
  
  /**
   * Removes the specified {link Mediator}
   * @param med The mediator to be removed from the mediatorList.
   */
  public void removeAddress(Addressable med) {
  	if (address == med) address = null;
  }

  /**
   * Return the value of the address reference
   */
  public final Addressable getAddress() {
    return address;
  }

  /**
   * Return the name of the actual Mediator class
   */
  public final String mediatorName() {
    String str =  String.valueOf(this);
    return str.substring(str.lastIndexOf('.')+1, str.lastIndexOf('@'));
  }

  /**
   * Forwards the specified signal to the Addressable object refered to by the
   * address reference.
   * It is assumed that the address of this Mediator has been given a value.
   * If not, a NullPointerException is thrown.
   */
  public void forward(Message sig) {
    address.forward(sig);
  }  

  /**
   * Used at the edge of the system to input the specified signal.
   * The signal will be forwarded into the system.
   * @param sig is assumed to be not null.
   *    If not, a IllegalArgumentException is thrown.
   */
  public final void input(Message sig) throws IllegalArgumentException {
    if (sig == null)
      throw new IllegalArgumentException("The Message argument cannot be null.");
    forward(sig);
  }
}
