package se.ericsson.eto.norarc.javaframe;
import java.util.*;

/**
 * a MultiCastMediator is a Mediator that replicates the received Message and
 * sends it to all its outgoing connections (which are organized in a list)
 * @see Message
 */
public class MultiCastMediator extends Mediator {
  public LinkedList mediatorList;

  /**
   * Creates a new MultiCastmediator object with a mediatorList.
   */
  public MultiCastMediator() {
    mediatorList = new LinkedList();
  }

  /**
   * Duplicates and forwards the specified signal to all the Mediator
   * objects refered to in the mediatorList.
   */
  public void forward(Message sig) {
    int i;
    Message newSig = null;
    int listSize = mediatorList.size();
    for (i=0; i<listSize; i++) {
      try {
        newSig = sig.duplicate();
      } catch (CloneNotSupportedException e)
      { System.err.println("Clone of Message not possible: "+sig.messageName());
        System.exit(0);
      }
      ((Addressable)mediatorList.get(i)).forward(newSig);
    }
  }

  /**
   * Adds the specified {link Mediator} to the mediatorList.
   * @param newMediator Assumed to be not null.
   *    If not, an IllegalArgumentException is thrown.
   */
  public void addAddress(Addressable newMediator) throws IllegalArgumentException {
    if (newMediator == null)
      throw new IllegalArgumentException("The Addressable argument cannot be null.");
    mediatorList.add(newMediator);
  }
  
  /**
   * Removes the specified {link Mediator} from the mediatorList.
   * @param med The mediator to be removed from the mediatorList.
   */
  public void removeAddress(Addressable med) {
    mediatorList.remove(med);
  }
}