package se.ericsson.eto.norarc.javaframe;

/**
 * The RouterMediator has a Router state machine associated. The router mediator will
 * merely let the associated Router do the actual routing
 * @see SimpleRouterMediator
 * @see Router
 */
public class RouterMediator extends Mediator {
  Router mediatorRouter; // the Router of the Mediator

  /**
   * Associates the given Router with this RouterMediator
   * @param router Assumed to be not null.
   *    If not, an IllegalArgumentException is thrown.
   */
  public RouterMediator(Router router)  throws IllegalArgumentException {
    if (router == null)
      throw new IllegalArgumentException("The Router argument of RouterMediator cannot be null.");
     mediatorRouter = router;
     address = router; // There is no reason that address should point to anything else
  }

  /**
   * @return reference to the associated Router state machine
   */
  public final Router getRouterMachine() {
    return mediatorRouter;
  }

  /**
   * Adds the specified Mediator to the mediatorList.
   * @param newMediator Assumed to be not null.
   *    If not, an IllegalArgumentException is thrown.
   */
  public final void addAddress(Addressable newMediator) throws IllegalArgumentException {
    if (newMediator == null)
      throw new IllegalArgumentException("The Addressable argument of addAddress cannot be null.");
    mediatorRouter.addAddress(newMediator);
  }

  /**
   * Forwards the specified signal to the Addressable object refered to by the
   * mediatorrouter reference.
   * Invariant: mediatorRouter != null
   */
  public final void forward(Message sig) {
    mediatorRouter.forward(sig);
  }
}