package se.ericsson.eto.norarc.javaframe;
import java.util.*;

/**
 * A SimpleRouterMediator is a superclass of Mediators that overrides the <code>forward</code> method
 * and thus defines how the routing should go.
 * As a default the SimpleRouterMediator has a list of Mediators to describe the potential addressing
 * targets
 */
public abstract class SimpleRouterMediator extends Mediator {
  public LinkedList mediatorList;

  public SimpleRouterMediator() {
    address = null;
    mediatorList = new LinkedList();
  }

  /**
   * Add another address to the list of addressing targets
   * @param newMediator Assumed to be not null.
   *    If not, an IllegalArgumentException is thrown.
   */
  public void addAddress(Addressable newMediator) throws IllegalArgumentException {
    if (newMediator == null)
      throw new IllegalArgumentException("The Addressable argument of addAddress cannot be null.");
    mediatorList.add(newMediator); // Adds the specified Mediator to the mediatorList.
  }
  
  /**
   * Removes the specified {link Mediator} from the mediatorList.
   * @param med The mediator to be removed from the mediatorList.
   */
  public void removeAddress(Addressable med) {
    mediatorList.remove(med);
  }

  /**
   * forward must be overridden by user application
   */
  public abstract void forward(Message sig);
}
