package se.ericsson.eto.norarc.javaframe;
import java.io.Serializable;

/**
 * The class for the singular States of the StateMachines.
 */
public class State implements Serializable {
  public CompositeState enclosingState = null; // the enclosing State of this State
  public String stateName; // The symbolic name of this State

  /**
   * State constructor with user supplied name.
   * @param sn The name of this State.
   */
  public State(String sn) {
    stateName = sn;
  }

  /**
   * State constructor with default name.
   */
  public State() {
    stateName = String.valueOf(this);
  }

  /**
   * The name of the this State.
   * @return The symbolic name this State.
   */
  public final String stateName() {
    return stateName;
  }
  
  /**
   * outputs a {@link Message} to a given (local) {@link Mediator}.
   * The method is to be used within the method enterState(),
   * execTrans(), or outofInnerCompositeState().
   * @param sig The Message to be sent.
   * @param receiver The Mediator that will receive (and forward) the Message.
   * @param curfsm The StateMachine that is conceptually performing this operation.
   */
  public final void output(Message sig, Mediator receiver, StateMachine curfsm) {
    if (curfsm.myScheduler.trc.trace_on)
      curfsm.myScheduler.trc.traceOutput(sig, receiver);
    receiver.forward(sig);
  }
  
  /**
   * Will be executed every time this State is entered.
   * Default is empty.
   * Note: not to be called from user code.
   */
  protected void entry(StateMachine curfsm) {
  }

  /**
   * Will be executed every time this State is exited.
   * Default is empty.
   * Note: not to be called from user code.
   */
  protected void exit(StateMachine curfsm) {
  }

  // ####################################################################
  // Utility to be used within the transitions (see the methods
  // enterState(), execTrans(), and outofInnerCompositeState() of
  // CompositeState)

  /**
   * Changes the current State of the StateMachine to this singular State.
   * Note: a singular State do not have an entry index.
   * Note: enterState should not be redefined for a singular State.
   * @param curfsm The StateMachine that gets the new singular State.
  */
  public void enterState(StateMachine curfsm) {
    // Move the signals in the saveQueue back to the front of the messageBox
    // in order to be reinspected in newState.
    curfsm.messageBox.moveMailBox(curfsm.saveQueue);
    curfsm.currentState = this;
    entry(curfsm);
    if (curfsm.myScheduler.trc.trace_on) // Trace the end of the transition
      curfsm.myScheduler.trc.traceNextState(curfsm.currentState);
  }
}
