package se.ericsson.eto.norarc.javaframe;
import java.util.Timer;
import java.util.TimerTask;

/**
 * A class to include Timers into the JavaFrame machinery.
 * The TimerMsg is a Message that is included in the appropriate Message queue
 * whenever the given time has elapsed.
 * A Timer must be created (and thus initiatied with a delay), and then it must be started.
 * A Timer may then time out in which case it is entered into the message queue.
 * A Timer may be stopped before it has timed out (or before it has been consumed). Then there will
 * be no sign of the TimerMsg.
 */
public class TimerMsg extends Message {
  protected static Timer jfTimer = new Timer();
  protected StateMachine timedStateMachine;
  private long timedelay;
  protected JFTimerTask jfTT = null;

  public class JFTimerTask extends TimerTask {
    TimerMsg thisMsg;

    public JFTimerTask(TimerMsg thisMsg) {
      super();
      this.thisMsg = thisMsg;
    }

    public void run() {
      jfTT = null;
      timedStateMachine.forward(thisMsg);
    }
  }

  /**
   * @param timedelay The delay in milliseconds of the timer from its start
   * @param myStateMachine the StateMachine that is associated with this TimerMsg
   */
  public TimerMsg(int timedelay, StateMachine myStateMachine) {
    // timedelay is in milliseconds
    super();
    timedStateMachine = myStateMachine;
    this.timedelay = timedelay;
  }

  /**
   * @param timeDelay The new delay in milliseconds
   */
  public void setDelay(int timeDelay) {
    // setting the timedelay in milliseconds
    timedelay = timeDelay;
  }

  /**
   * Starts the timer with current delay
   */
  public void startTimer() {
    if (jfTT != null) jfTT.cancel();
    jfTT = new JFTimerTask(this);
    jfTimer.schedule(jfTT, timedelay);
  }

  /**
   * Starts the timer with a new delay
   * @param timeDelay New delay in milliseconds
   */
  public void startTimer(int timeDelay) {
    // setting the timers delay and starting it
    timedelay = timeDelay;
    startTimer();
  }

  /**
   * Stops the Timer.
   * It will also be removed from the message queue if it is already included there
   */
  public void stopTimer() {
    if (jfTT != null) jfTT.cancel();
    // Then we remove this TimerMsg from the message queue if it is already in there
    // (that is according to SDL definitions of timers)
    timedStateMachine.messageBox.remove(this);
  }
}