/*
 * Created on 10.apr.2005
 *
 * Gran, 
 */
package smsmediators;

import se.ericsson.eto.norarc.javaframe.Mediator;
import se.ericsson.eto.norarc.javaframe.Message;
import no.uio.ifi.pats.client.positioning.GsmPositioner;
import no.uio.ifi.pats.client.positioning.GsmPositionerException;
import no.uio.ifi.pats.client.sms.SmsSender;
import no.uio.ifi.pats.client.sms.SmsSenderException;
import no.uio.ifi.pats.client.sms.SmsMessage;

import java.util.*;

/**
 * Purpose: The SmsOutputMediator receives an object of type Sms and sends it as a "real" SMS.
 * If the Sms from the system is a positioning request (Sms constructor with 1 param[staticId] used)
 *  then a positioning request is sent. 
 */
public class SmsOutputMediator extends Mediator {
	
	private SmsSender smsSender;
	private GsmPositioner gsmPositioner;
	private SmsMessage smsMessage;
	private SmsMessage smsPosMessage;
	
	private final static String smsPath = "http://pats.ifi.uio.no:6667/pats/Http2Sms";
	private final static String posPath = "http://pats.ifi.uio.no:6667/pats/GsmPositioner";
	private final static String gatewayId = "patsIfi";
	/**
	 * @param username The username to use for sending SMS messages
	 * @param password The password to use for sending SMS messages
	 */
	public SmsOutputMediator(String username, String password) 
	{
		smsSender = new SmsSender(smsPath, username, password, gatewayId);
		gsmPositioner = new GsmPositioner(posPath, username, password);
	}
	
	public void forward(Message sig) {
		
		if(sig instanceof Sms) {
			
			try {
				
				// We ask the sms sender to send the message
				smsSender.send((SmsMessage)((Sms)sig).getSmsMessage());
				
			} catch(SmsSenderException e) {
				// If something goes wrong during the sender, the SmsSender throws a SmsSenderException
				System.err.println("SmsSenderException at "+new Date().toString()); // before: e.printStackTrace();
			}	
		} else {
			
			try {
				PosRequest posRequest = (PosRequest) sig;
				gsmPositioner.sendPositionRequest(posRequest.getPositioningId(), posRequest.getMessageId());
			} catch (GsmPositionerException e) {
				System.err.println("GsmPositionerException at "+new Date().toString()); // before: e.printStackTrace();
			}
			
		}
	}
}
