package no.uio.ifi.uml.sedi;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.EventObject;
import java.util.List;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Font;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.draw2d.FigureCanvas;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.resource.ImageDescriptor;

import org.eclipse.gef.ContextMenuProvider;
import org.eclipse.gef.DefaultEditDomain;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.KeyHandler;
import org.eclipse.gef.KeyStroke;
import org.eclipse.gef.Tool;

import org.eclipse.gef.palette.ConnectionCreationToolEntry;
import org.eclipse.gef.palette.CreationToolEntry;
import org.eclipse.gef.palette.MarqueeToolEntry;
import org.eclipse.gef.palette.PaletteContainer;
import org.eclipse.gef.palette.PaletteDrawer;
import org.eclipse.gef.palette.PaletteEntry;
import org.eclipse.gef.palette.PaletteGroup;
import org.eclipse.gef.palette.PaletteRoot;
import org.eclipse.gef.palette.SelectionToolEntry;
import org.eclipse.gef.palette.ToolEntry;

import org.eclipse.gef.requests.CreationFactory;
import org.eclipse.gef.requests.SimpleFactory;

import org.eclipse.gef.ui.actions.DeleteAction;
import org.eclipse.gef.ui.actions.DirectEditAction;
import org.eclipse.gef.ui.actions.EditorPartAction;
import org.eclipse.gef.ui.actions.GEFActionConstants;
import org.eclipse.gef.ui.actions.SaveAction;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.gef.ui.actions.StackAction;
import org.eclipse.gef.ui.parts.GraphicalEditorWithPalette;
import org.eclipse.gef.ui.parts.GraphicalViewerKeyHandler;

import no.uio.ifi.uml.sedi.action.LifelineDecomposeAction;
import no.uio.ifi.uml.sedi.edit.SeDiEditPartFactory;
import no.uio.ifi.uml.sedi.uml2.CreateUML2ToolEntry;
import no.uio.ifi.uml.sedi.model.*;



public class SeDiEditor extends GraphicalEditorWithPalette {

	private SeDi diagram;
	private KeyHandler sharedKeyHandler;
	//private PaletteRoot root;
	private boolean isDirty=false;
	
	public SeDiEditor(){
		setEditDomain(new DefaultEditDomain(this));
	}
	
	protected void configureGraphicalViewer(){
		super.configureGraphicalViewer();
		
		final GraphicalViewer viewer = getGraphicalViewer();
		viewer.setEditPartFactory(new SeDiEditPartFactory());
	
		// Register actions
		addSelectionAction(new DeleteAction((IWorkbenchPart)this));
		addSelectionAction(new DirectEditAction((IWorkbenchPart)this));
		addPropertyAction(new SaveAction(this));
		addSelectionAction(new LifelineDecomposeAction((IWorkbenchPart)this));

		// Register context menu
		ContextMenuProvider provider = new SeDiContextMenuProvider(viewer, getActionRegistry());
		viewer.setContextMenu(provider);
		getSite().registerContextMenu("no.uio.ifi.uml.sedi.context", provider, viewer);
		
		// Register key handler
		viewer.setKeyHandler(
			new GraphicalViewerKeyHandler(viewer)
				.setParent(getCommonKeyHandler()));
		
		((FigureCanvas) viewer.getControl()).setScrollBarVisibility(FigureCanvas.ALWAYS);
		viewer.getControl().setFont(FontRegistry.getFont());
	}
	
	public void dispose() {
		FontRegistry.releaseFont();
		super.dispose();
	}

	protected void addSelectionAction(SelectionAction action) {
		getActionRegistry().registerAction(action);
		getSelectionActions().add(action.getId());		
	}
	protected void addStackAction(StackAction action) {
		getActionRegistry().registerAction(action);
		getStackActions().add(action.getId());		
	}
	protected void addPropertyAction(EditorPartAction action) {
		getActionRegistry().registerAction(action);
		getPropertyActions().add(action.getId());		
	}
	
	protected KeyHandler getCommonKeyHandler(){
		if (sharedKeyHandler == null){
			sharedKeyHandler = new KeyHandler();
			
			IAction action = getActionRegistry().getAction(ActionFactory.DELETE.getId()); 
			sharedKeyHandler.put(
				KeyStroke.getPressed(SWT.DEL, 127, 0),
				action);
			sharedKeyHandler.put(
				KeyStroke.getPressed(SWT.F2, 0),
				getActionRegistry().getAction(GEFActionConstants.DIRECT_EDIT));
		}
		return sharedKeyHandler;
	}
	
	protected void initializeGraphicalViewer() {
		//initializeActionRegistry(); FRANKD Called during init
		getGraphicalViewer().setContents(diagram);
	}

	protected void setInput(IEditorInput input) {
		super.setInput(input);

		IFile file = ((IFileEditorInput) input).getFile();
		InputStream in = null;
		try	{
			setPartName(file.getName());
			InputStream is = file.getContents(true);
			in = is;
			ObjectInputStream ois = new ObjectInputStream(is);
			in = ois;
			diagram = (SeDi) ois.readObject();
			ois.close();
			in = null;
		}
		catch (Exception e) {
			SeDiPlugin.getDefault().getLog().log(
				new Status(IStatus.ERROR, 
					SeDiPlugin.getDefault().getBundle().getSymbolicName(), 
					100, "Error loading diagram: " + file.getName(),
					e));
			diagram = new SeDi();
		}
		finally {
			if (in != null) {
				try {
					in.close();
				}
				catch (Exception e) {
					e.printStackTrace(System.err);
				}
			}
		}
	}
	
	public void doSave(IProgressMonitor monitor) {
		IFile file =((IFileEditorInput)getEditorInput()).getFile();
		OutputStream out = null;
		try {
			ByteArrayOutputStream byteOut = new ByteArrayOutputStream();
			out = byteOut;
			ObjectOutputStream objectOut =	new ObjectOutputStream(byteOut);
			out = objectOut;
			objectOut.writeObject(diagram);
			objectOut.close();
			out = null;
			file.setContents(new ByteArrayInputStream(byteOut.toByteArray()), true, false, monitor);
			getCommandStack().markSaveLocation();
		} 
		catch (Exception e) {
			SeDiPlugin.getDefault().getLog().log(
				new Status(IStatus.ERROR, 
					SeDiPlugin.getDefault().getBundle().getSymbolicName(), 
					100, "Error saving diagram: " + file.getName(),
					e));
			// TODO Is it sensible to reset the diagram here? -Frank
			diagram = new SeDi();
		}
		finally {
			if (out != null) {
				try {
					out.close();
				}
				catch (Exception e) {
					e.printStackTrace(System.err);
				}
			}
		}
	}
	
	public boolean isDirty() {
		return isDirty;
	}
		
	protected void setDirty(boolean dirty) {
		if (isDirty != dirty) {
			isDirty = dirty;
			firePropertyChange(IEditorPart.PROP_DIRTY);
		}
	}
	
	public void commandStackChanged(EventObject event) {
		setDirty(getCommandStack().isDirty());
		super.commandStackChanged(event);
	}

	
	public void doSaveAs() {
	}
	
	public SeDi getDiagram(){
		return diagram;
	}
	
	protected PaletteRoot getPaletteRoot() {
		PaletteRoot root = new PaletteRoot();
		PaletteGroup group = new PaletteGroup("Tools");
		root.add(group);
				
		PaletteDrawer drawer = new PaletteDrawer("SeDi", ImageDescriptor.createFromFile(Frame.class, "icons/seq.GIF"));
		drawer.setUserModificationPermission(PaletteEntry.PERMISSION_FULL_MODIFICATION);
		drawer.setInitialState(PaletteDrawer.INITIAL_STATE_PINNED_OPEN);
		
		List entries = new ArrayList();
		
		ToolEntry tool = new SelectionToolEntry();
		tool.setLabel("Selection");
		entries.add(tool);
		root.setDefaultEntry(tool);
		
		tool = new MarqueeToolEntry();
		tool.setLabel("Group");
		entries.add(tool);
		drawer.addAll(entries);
		drawer.addAll(createFigureDrawerElements());
		
		group.add(drawer);
		group.add(createMessageDrawer());
		group.add(createUML2Drawer());
		return root;
	}

	private List createFigureDrawerElements(){
		List entries = new ArrayList();
		
		PaletteEntry entry = new CreationToolEntry(
				"Frame", 
				"Add a new Frame to the diagram",
				new SimpleFactory(Frame.class),
				ImageDescriptor.createFromFile(Frame.class, "icons/interaction.GIF"),
				ImageDescriptor.createFromFile(Frame.class, "icons/interaction.GIF")
			);
		entries.add(entry);
		
		entry = new CreationToolEntry(
				"Lifeline", 
				"Add a new Lifeline to the diagram",
				new SimpleFactory(Lifeline.class),
				ImageDescriptor.createFromFile(Frame.class, "icons/Lifeline.GIF"),
				ImageDescriptor.createFromFile(Frame.class, "icons/Lifeline.GIF")
			);
		entries.add(entry);
		
		entry = new CreationToolEntry(
				"CF", 
				"Add a new CombinedFragment to the diagram",
				new SimpleFactory(CF.class),
				ImageDescriptor.createFromFile(Frame.class, "icons/cf.GIF"),
				ImageDescriptor.createFromFile(Frame.class, "icons/cf.GIF")
			);
		entries.add(entry);
		
		entry = new ConnectionCreationToolEntry2(
				"Separator",
				"Add a separator to a combined fragment",
				new MessageFactory("s"),
				ImageDescriptor.createFromFile(Frame.class, "icons/separator.GIF"),
				ImageDescriptor.createFromFile(Frame.class, "icons/separator.GIF")
			); 
		entries.add(entry);
		
		entry = new CreationToolEntry(
				"InteractionUse", 
				"Add a new InteractionUse to the diagram",
				new SimpleFactory(IO.class),
				ImageDescriptor.createFromFile(Frame.class, "icons/iu.GIF"),
				ImageDescriptor.createFromFile(Frame.class, "icons/iu.GIF")
			);
		entries.add(entry);
		
		entry = new CreationToolEntry(
				"Coregion", 
				"Add a new Coregion to the diagram",
				new SimpleFactory(Coregion.class),
				ImageDescriptor.createFromFile(Frame.class, "icons/coregion.GIF"),
				ImageDescriptor.createFromFile(Frame.class, "icons/coregion.GIF")
			);
		entries.add(entry);
		
		entry = new CreationToolEntry(
				"EO", 
				"Add a new ExecutionOccurence to the diagram",
				new SimpleFactory(EO.class),
				ImageDescriptor.createFromFile(Frame.class, "icons/eo.GIF"),
				ImageDescriptor.createFromFile(Frame.class, "icons/eo.GIF")
			);
		entries.add(entry);
		
		entry = new CreationToolEntry(
				"Termination",
				"Add a new Termination to the diagram",
				new SimpleFactory(Terminate.class),
				ImageDescriptor.createFromFile(Frame.class, "icons/terminate.GIF"),
				ImageDescriptor.createFromFile(Frame.class, "icons/terminate.GIF")
			);
		entries.add(entry);
		
		entry = new CreationToolEntry(
				"Continuation", 
				"Add a new Continuation to the diagram",
				new SimpleFactory(Continuation.class),
				ImageDescriptor.createFromFile(Frame.class, "icons/continuation.gif"),
				ImageDescriptor.createFromFile(Frame.class, "icons/continuation.gif")
			);
		entries.add(entry);
		
		
		return entries;
	}

	private PaletteContainer createMessageDrawer(){
		PaletteDrawer drawer = new PaletteDrawer("Messages", ImageDescriptor.createFromFile(Frame.class, "icons/msgbanner.GIF"));
		drawer.setUserModificationPermission(PaletteEntry.PERMISSION_FULL_MODIFICATION);
		drawer.setInitialState(PaletteDrawer.INITIAL_STATE_PINNED_OPEN);
		
		List entries = new ArrayList();

		ToolEntry tool = new ConnectionCreationToolEntry2(
			"Asynchronous",
			"Asynchronous",
			new MessageFactory("a"),
			ImageDescriptor.createFromFile(Frame.class, "icons/asmessage.GIF"),
			ImageDescriptor.createFromFile(Frame.class, "icons/asmessage.GIF")
		);
		entries.add(tool);
		
		tool = new ConnectionCreationToolEntry2(
			"Call",
			"Call",
			new MessageFactory("c"),
			ImageDescriptor.createFromFile(Frame.class, "icons/callmessage.GIF"),
			ImageDescriptor.createFromFile(Frame.class, "icons/callmessage.GIF")
		);
		entries.add(tool);
		
		tool = new ConnectionCreationToolEntry2(
			"Reply",
			"Reply",
			new MessageFactory("r"),
			ImageDescriptor.createFromFile(Frame.class, "icons/replymessage.GIF"),
			ImageDescriptor.createFromFile(Frame.class, "icons/replymessage.GIF")
		);
		entries.add(tool);
		
		drawer.addAll(entries);
		return drawer;
	}

	// The tool palette has been changed between Eclipse <= 3.0/RSM  
	// and Eclipse >= 3.1.
	//
	// We want connection tools to be handled somewhat different from the
	// default configuration. Firstly, we want the tool to be automatically
	// deselected after it has been used once, just like the object tools.
	// Secondly, we want the newly created connection to be automatically
	// selected.
	// 
	// In 3.1 a call to
	//    toolEntry.setToolProperty(
	//       AbstractTool.PROPERTY_UNLOAD_WHEN_FINISHED, Boolean.TRUE);
	// is sufficient to achieve the first feature. The second feature
	// can similarly (in 3.1) be programmed by calling
	//    toolEntry.setToolClass(SeDiConnectionCreationTool.class)
	//
	// Neither of these mechanims are available in earlier version of GEF 
	// so we have to subclass and override methods.
	private class ConnectionCreationToolEntry2 extends ConnectionCreationToolEntry {
		public ConnectionCreationToolEntry2(
				String label, String shortDesc, CreationFactory factory,
				ImageDescriptor iconSmall, ImageDescriptor iconLarge) {
			super(label, shortDesc, factory, iconSmall, iconLarge);
			// 3.1: setToolClass(SeDiConnectionCreationTool.class);
		}

		// 3.0:
		public Tool createTool() {
			return new SeDiConnectionCreationTool(factory);
		}
	}
	
	private PaletteContainer createUML2Drawer(){
		PaletteDrawer drawer = new PaletteDrawer("Create UML2", ImageDescriptor.createFromFile(Frame.class, "icons/uml2.gif"));
		drawer.setUserModificationPermission(PaletteEntry.PERMISSION_FULL_MODIFICATION);
		drawer.setInitialState(PaletteDrawer.INITIAL_STATE_PINNED_OPEN);
		
		ToolEntry tool = new CreateUML2ToolEntry(
				"UML2", 
				"Generate the UML2 meta-model",
				this,
				getEditDomain(),
				ImageDescriptor.createFromFile(Frame.class, "icons/uml2gen.gif"),
				ImageDescriptor.createFromFile(Frame.class, "icons/uml2gen.gif")
			);
		drawer.add(tool);
		return drawer;
	}
	
	public boolean isSaveAsAllowed() {
		return false;
	}
}


class FontRegistry {
	private static Font font;
	private static int count;
	
	/**
	 * Returns the best default font, or null if none of the desired
	 * fonts could be found on the system. In that case, the system
	 * default font should take effect.
	 */
	public static Font getFont() {
		if (count++ == 0) {
			String[] families = { "Tahoma", "Arial" };
			for (int i = 0; i < families.length && font == null; ++i) {
				try {
					font = new Font(null, families[i], 8, SWT.NORMAL);
				}
				catch (Throwable e) {
					// ignore and try next font
				}
			}
		}
		return font;
	}
	
	public static void releaseFont() {
		if (--count == 0 && font != null) {
			font.dispose();
			font = null;
		}
	}
}
