package no.uio.ifi.uml.sedi;

import java.util.*;

import org.osgi.framework.BundleContext;
import org.osgi.framework.Constants;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.cheatsheets.OpenCheatSheetAction;
import org.eclipse.ui.plugin.*;

/**
 * The main plugin class to be used in the desktop.
 */
public class SeDiPlugin extends AbstractUIPlugin {
	//The shared instance.
	private static SeDiPlugin plugin;
	//Resource bundle.
	private ResourceBundle resourceBundle;
	
	// This extension is used to identify sedi model files
	public static final String FILE_EXTENSION = "sd2";
	
	/**
	 * The constructor.
	 */
	public SeDiPlugin() {
		super();
		plugin = this;
		try {
			resourceBundle = ResourceBundle.getBundle("SeDiPluginResources");
		} catch (MissingResourceException x) {
			resourceBundle = null;
		}
	}

	/**
	 * This method is called upon plug-in activation
	 */
	public void start(final BundleContext context) throws Exception {
		super.start(context);

		// If this is the first time the plugin is started, we want to
		// open the "Getting Started" cheat-sheet. This has to be
		// done from the UI thread, so we have to go via the Display
		// and post a Runnable on its event queue.
		IPreferenceStore prefStore = getPreferenceStore();
		String version = (String) getBundle().getHeaders().get(Constants.BUNDLE_VERSION);
		String lastStart = prefStore.getString("LastStartVersion");
		if (!lastStart.equals(version))
			prefStore.setValue("LastStartVersion", version);
		
		if (lastStart.length() == 0) {
			// TODO Remove the autostart extension - is it possible?
			//      The ExtensionRegistry is readonly. It is maybe possible
			//      to edit the plugin.xml file directly.

			IWorkbench workbench = PlatformUI.getWorkbench();
			workbench.getDisplay().asyncExec(new Runnable() {
				public void run() {
					try {
						new OpenCheatSheetAction("no.uio.if.uml.sedi.gettingstarted").run();
					}
					catch (Exception e) {
						// It's not unthinkable for this to happen. For example, 
						// the documentation plugin might not be installed.
						// It's hardly critical but we log an information message.
						getLog().log(
							new Status(IStatus.INFO, getBundle().getSymbolicName(),
									100, "Failed to open GettingStarted cheat sheet.", e));				
					}
				}
			});
		}		
	}

	/**
	 * This method is called when the plug-in is stopped
	 */
	public void stop(BundleContext context) throws Exception {
		super.stop(context);
	}

	/**
	 * Returns the shared instance.
	 */
	public static SeDiPlugin getDefault() {
		return plugin;
	}

	/**
	 * Returns the string from the plugin's resource bundle,
	 * or 'key' if not found.
	 */
	public static String getResourceString(String key) {
		ResourceBundle bundle = SeDiPlugin.getDefault().getResourceBundle();
		try {
			return (bundle != null) ? bundle.getString(key) : key;
		} catch (MissingResourceException e) {
			return key;
		}
	}

	/**
	 * Returns the plugin's resource bundle,
	 */
	public ResourceBundle getResourceBundle() {
		return resourceBundle;
	}
}
