package no.uio.ifi.uml.sedi.action;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.CompoundCommand;
import org.eclipse.gef.requests.GroupRequest;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.ui.IWorkbenchPart;

import no.uio.ifi.uml.sedi.edit.LifelineEditPart;
import no.uio.ifi.uml.sedi.model.Lifeline;

public class LifelineDecomposeAction extends SelectionAction {

	public static final String ID = "no.uio.ifi.uml.sedi.decompose";
	
	public LifelineDecomposeAction(IWorkbenchPart part, int style) {
		super(part, style);
	}

	public LifelineDecomposeAction(IWorkbenchPart part) {
		super(part);
	}	
	
	public void init() {
		super.init();
		setId(ID); 
		setText("Decompose lifecycle");
	}

	protected boolean calculateEnabled() {
		final List selectedObjects = getSelectedObjects();
		if (selectedObjects.isEmpty())
			return false;
		
		final Iterator iterator = selectedObjects.iterator();
		boolean decompose = false; 
		while (iterator.hasNext()) {
			final Object editPart = iterator.next();
			if (!(editPart instanceof LifelineEditPart))
				return false;
			final LifelineEditPart lifelinePart = (LifelineEditPart) editPart;
			final Lifeline lifeline = (Lifeline) lifelinePart.getModel();
			decompose |= lifeline.isDecomposed();
		}
		
		setChecked(decompose);
		return true;
	}

	public void run() {
		execute(createDecomposeCommand(getSelectedObjects()));
	}

	protected Command createDecomposeCommand(final List objects) {
		// verify that the command makes sense
		if (objects.isEmpty())
			return null;
		for (int i = 0; i < objects.size(); ++i) {
			if (!(objects.get(i) instanceof EditPart))
				return null;
		}

		final GroupRequest decomposeReq =
			new GroupRequest("no.uio.ifi.uml.sedi.DecomposeLifelineRequest");
		decomposeReq.setEditParts(objects);
		final Map data = new HashMap();
		data.put("decompose", isChecked() ? Boolean.FALSE : Boolean.TRUE);
		decomposeReq.setExtendedData(data);

		final CompoundCommand compoundCmd = new CompoundCommand();
		for (int i = 0; i < objects.size(); i++) {
			EditPart object = (EditPart) objects.get(i);
			Command cmd = object.getCommand(decomposeReq);
			if (cmd != null) compoundCmd.add(cmd);
		}

		return compoundCmd;
	}
	/*
		final Iterator iterator = selectedObjects.iterator();
		while (iterator.hasNext()) {
			final LifelineEditPart lifelinePart = (LifelineEditPart) iterator.next();
			final Lifeline lifeline = (Lifeline) lifelinePart.getModel();
			
			if (decompose) {
				if (lifeline.getRef() == null)
					lifeline.setRef("Your text here");
			}
			else
				lifeline.setRef(null);
		}
	}
	*/
}
