
package no.uio.ifi.uml.sedi.edit;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Rectangle;

import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.NodeEditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.RequestConstants;
import org.eclipse.gef.requests.DirectEditRequest;

import org.eclipse.jface.viewers.TextCellEditor;

import no.uio.ifi.uml.sedi.model.Lifeline;
import no.uio.ifi.uml.sedi.figures.LifelineFigure;
import no.uio.ifi.uml.sedi.edit.manager.LifelineEditManager;
import no.uio.ifi.uml.sedi.edit.manager.LifelineCellEditorLocator;
import no.uio.ifi.uml.sedi.edit.policy.SeDiNodeEditPolicy;
import no.uio.ifi.uml.sedi.edit.policy.SeDiSubpartEditPolicy;
import no.uio.ifi.uml.sedi.edit.policy.LifelineEditPolicy;


public class LifelineEditPart extends SeDiSubpartEditPart implements NodeEditPart{
	
	//private DirectEditManager manager;
	
	protected IFigure createFigure() {
		LifelineFigure life = new LifelineFigure();
		return life;
	}

	protected void createEditPolicies() {
		installEditPolicy(EditPolicy.DIRECT_EDIT_ROLE, new LifelineEditPolicy());
		installEditPolicy(EditPolicy.GRAPHICAL_NODE_ROLE, new SeDiNodeEditPolicy());
		installEditPolicy(EditPolicy.COMPONENT_ROLE, new SeDiSubpartEditPolicy());
		//installEditPolicy(EditPolicy.LAYOUT_ROLE,new SeDiXYLayoutEditPolicy());
	}
	
	protected void refreshVisuals(){
		Lifeline life = (Lifeline)getModel();
		LifelineFigure lifeFigure=(LifelineFigure)getFigure();
		lifeFigure.setText(life.getText());
		lifeFigure.setRefText(life.getRef());
		
		Rectangle r = new Rectangle(life.getLocation().x,life.getLocation().y,life.getSize().width,life.getSize().height);
		((GraphicalEditPart)getParent()).setLayoutConstraint(this,lifeFigure,r);
		lifeFigure.layoutAnchors(life.getSize().width, life.getSize().height);
		
		//refreshChildren();
	}
	
	/* protected List getModelChildren(){
		return ((SeDiSubpart)getModel()).getAllChildren();
	} */
	
	public void performRequest(Request request){
		if (request.getType() == RequestConstants.REQ_DIRECT_EDIT)
			performDirectEdit(request);
		else
			super.performRequest(request);
	}

	private void performDirectEdit(final Request request) {
		final LifelineFigure figure = (LifelineFigure)getFigure();
		
		final int target;
		if (request instanceof DirectEditRequest) {
			DirectEditRequest deRequest = (DirectEditRequest) request;
			target = figure.findEditTargetAt(deRequest.getLocation());
		}
		else
			target = figure.findEditTargetAt(null);
		
		final LifelineEditManager manager = 
			new LifelineEditManager(
				this, 
				TextCellEditor.class, 
				new LifelineCellEditorLocator(figure, target), 
				target);
		manager.show();
	}
	
	/*private void performDecompose(final Request request) {
		
	}*/
}
