package no.uio.ifi.uml.sedi.edit;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.List;

import org.eclipse.draw2d.Connection;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.MidpointLocator;
import org.eclipse.draw2d.PolygonDecoration;
import org.eclipse.draw2d.PolylineConnection;
import org.eclipse.draw2d.PolylineDecoration;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.Request;
import org.eclipse.gef.RequestConstants;
import org.eclipse.gef.editparts.AbstractConnectionEditPart;
import org.eclipse.gef.tools.DirectEditManager;
import org.eclipse.jface.viewers.TextCellEditor;

import no.uio.ifi.uml.sedi.edit.manager.MessageCellEditorLocator;
import no.uio.ifi.uml.sedi.edit.manager.MessageEditManager;
import no.uio.ifi.uml.sedi.edit.policy.MessageEditPolicy;
import no.uio.ifi.uml.sedi.edit.policy.MessageEndpointEditPolicy;
import no.uio.ifi.uml.sedi.edit.policy.SeDiSubpartDirectEditPolicy;
import no.uio.ifi.uml.sedi.model.AMessage;
import no.uio.ifi.uml.sedi.model.CMessage;
import no.uio.ifi.uml.sedi.model.Message;
import no.uio.ifi.uml.sedi.model.RMessage;
import no.uio.ifi.uml.sedi.model.SeDiSubpart;
import no.uio.ifi.uml.sedi.model.Separator;

public class MessageEditPart 
	extends AbstractConnectionEditPart 
	implements PropertyChangeListener {

	private DirectEditManager manager;

	protected void createEditPolicies() {
		installEditPolicy(EditPolicy.CONNECTION_ENDPOINTS_ROLE,	new MessageEndpointEditPolicy());
		installEditPolicy(EditPolicy.CONNECTION_ROLE, new MessageEditPolicy());
		installEditPolicy(EditPolicy.DIRECT_EDIT_ROLE, new SeDiSubpartDirectEditPolicy());
	}

	/*
	protected IFigure createFigure() {
		PolylineConnection msg = new PolylineConnection();
		Label label = new Label(((Message) getModel()).getText());
		MidpointLocator ml = new MidpointLocator(msg, 0);
		ml.setRelativePosition(PositionConstants.NORTH);
		ml.setGap(3);
		msg.add(label, ml);
		Message model = (Message) getModel();
		if (model == null)
			return null;
		if (model instanceof AMessage) {
			msg.setTargetDecoration(new PolylineDecoration());
		} else if (model instanceof CMessage) {
			msg.setTargetDecoration(new PolygonDecoration());
		} else if (model instanceof RMessage) {
			msg.setLineStyle(Graphics.LINE_DASH);
			msg.setTargetDecoration(new PolylineDecoration());
		} else if (model instanceof Separator) {
			msg.setLineStyle(Graphics.LINE_DASH);
			msg.removeAll();
		}
		return msg;
	}
	*/
	
	protected IFigure createFigure() {
		Message model = (Message) getModel();
		if (model == null)
			return null;

		PolylineConnection figure = new PolylineConnection();

		// set a suitable locator
		final MidpointLocator locator;
		if (model instanceof Separator) {
			locator = new ConstraintLocator(figure, 0);
			locator.setRelativePosition(PositionConstants.SOUTH_EAST);
		}
		else {
			locator = new MidpointLocator(figure, 0);
			locator.setRelativePosition(PositionConstants.NORTH);
		}
		locator.setGap(3);
		figure.add(new Label(model.getText()), locator);
		
		// set a suitable target decoration
		if (model instanceof AMessage)
			figure.setTargetDecoration(new PolylineDecoration());
		else if (model instanceof CMessage)
			figure.setTargetDecoration(new PolygonDecoration());
		else if (model instanceof RMessage) {
			figure.setLineStyle(Graphics.LINE_DASH);
			figure.setTargetDecoration(new PolylineDecoration());
		} 
		else if (model instanceof Separator)
			figure.setLineStyle(Graphics.LINE_DASH);
				
		return figure;
	}
	
	private static class ConstraintLocator extends MidpointLocator {
		public ConstraintLocator(Connection c, int i) {
			super(c, i);
		}
		protected Point getReferencePoint() {
			Connection conn = getConnection();
			Point p1 = conn.getPoints().getPoint(getIndex());
			Point p2 = conn.getPoints().getPoint(getIndex() + 1);
			// Choose the one on the left - because we don't know 
			// if the message(Separator) was created left-right or
			// right-left.
			if (p2.x < p1.x)
				p1 = p2;
			conn.translateToAbsolute(p1);
			Point p = Point.SINGLETON;
			p.x = p1.x + 10;
			p.y = p1.y;
			return p;			
		}		
	}

	private void performDirectEdit() {
		if (manager == null)
			manager = new MessageEditManager(
				this, TextCellEditor.class,
				new MessageCellEditorLocator((PolylineConnection) getFigure()));
		manager.show();
	}

	public void performRequest(Request request) {
		if (request.getType() == RequestConstants.REQ_DIRECT_EDIT)
			performDirectEdit();
	}

	public void propertyChange(PropertyChangeEvent evt) {
		String prop = evt.getPropertyName();
		if (prop.equals("DirectEdit")) {
			PolylineConnection msg = (PolylineConnection) getFigure();
			List children = msg.getChildren();
			Label label = (Label) children.get(0);
			label.setText(((Message) getModel()).getText());
		}
		refresh();
	}

	public void activate() {
		if (isActive() == false) {
			super.activate();
			((SeDiSubpart) getModel()).addPropertyChangeListener(this);
		}
	}

	public void deactivate() {
		if (isActive()) {
			super.deactivate();
			((SeDiSubpart) getModel()).removePropertyChangeListener(this);
		}
	}
}
