package no.uio.ifi.uml.sedi.edit.policy;

import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.PolylineConnection;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.NodeEditPart;
import org.eclipse.gef.editpolicies.ConnectionEndpointEditPolicy;
import org.eclipse.gef.editpolicies.FeedbackHelper;
import org.eclipse.gef.requests.ReconnectRequest;

import no.uio.ifi.uml.sedi.edit.MessageEditPart;
import no.uio.ifi.uml.sedi.edit.SeDiSubpartEditPart;
import no.uio.ifi.uml.sedi.model.command.MessageCommand;

public class MessageEndpointEditPolicy extends ConnectionEndpointEditPolicy{

	protected void addSelectionHandles() {
		super.addSelectionHandles();
		getConnectionFigure().setLineWidth(2);
	}
	
	protected PolylineConnection getConnectionFigure(){
		return (PolylineConnection)((GraphicalEditPart)getHost()).getFigure();
	}
	
	protected void removeSelectionHandles(){
		super.removeSelectionHandles();
		getConnectionFigure().setLineWidth(0);
	}
	
	// Enhanced feedback for move message ------------------------------------
	
	private ConnectionAnchor originalSourceAnchor;
	private ConnectionAnchor originalTargetAnchor;
	private SeDiFeedbackHelper sourceFeedbackHelper;
	
	/**
	 * Lazily creates and returns the feedback helper for the given request. The helper will
	 * be configured as either moving the source or target end of the connection.
	 * @param request the reconnect request
	 * @return the feedback helper
	 */
	protected FeedbackHelper getFeedbackHelper(ReconnectRequest request) {
		if (request.isMovingStartAnchor()) {
			if (sourceFeedbackHelper == null) {
				sourceFeedbackHelper = new SeDiFeedbackHelper();
				sourceFeedbackHelper.setConnection(getConnection());
			}
			return sourceFeedbackHelper;			
		}
		else
			return super.getFeedbackHelper(request);
	}

	/**
	 * Shows or updates connection move feedback.  Called whenever a show feedback request is
	 * received for reconnection.
	 * @param request the reconnect request
	 */
	protected void showConnectionMoveFeedback(ReconnectRequest request) {
		if (request.isMovingStartAnchor()) {
			NodeEditPart sourceNode = null;
			if (request.getTarget() instanceof NodeEditPart)
				sourceNode = (NodeEditPart)request.getTarget();
			if (originalSourceAnchor == null) {
				originalSourceAnchor = getConnection().getSourceAnchor();
				originalTargetAnchor = getConnection().getTargetAnchor();
			}
			ConnectionAnchor sourceAnchor = null, targetAnchor = null;
			if (sourceNode != null) {
				MessageCommand cmd = (MessageCommand) sourceNode.getCommand(request);
				if (cmd != null) {
					MessageEditPart host = (MessageEditPart) getHost();
					
					// Find the feedback anchor for the source
					String ref = cmd.getSourceRef();
					sourceAnchor = cmd.getSourceAnchor();
					/*
					SeDiSubpartEditPart sourcePart = (SeDiSubpartEditPart) host.getSource();
					if (sourcePart.getModel() == cmd.getSource())
						sourceAnchor = sourcePart.getSourceConnectionAnchor(ref);
					else if (sourcePart.getParent().getModel() == cmd.getSource())
						sourceAnchor = ((SeDiSubpartEditPart)sourcePart.getParent()).getSourceConnectionAnchor(ref);
					else sourcePart.
						sourceAnchor = originalSourceAnchor;*/

					//sourceAnchor = sourceNode.getSourceConnectionAnchor(request);
					ref = cmd.getTargetRef();
					if (ref != null) {
						SeDiSubpartEditPart targetPart = (SeDiSubpartEditPart) host.getTarget();
						targetAnchor = targetPart.getTargetConnectionAnchor(ref);
					}
				}
				if (sourceAnchor == null)
					sourceAnchor = sourceNode.getSourceConnectionAnchor(request);;				
				if (targetAnchor == null)
					targetAnchor = originalTargetAnchor;				
			}
			else {
				sourceAnchor = originalSourceAnchor;
				targetAnchor = originalTargetAnchor;
			}
			SeDiFeedbackHelper helper = (SeDiFeedbackHelper) getFeedbackHelper(request);
			helper.update(sourceAnchor, targetAnchor);
		}
		else
			super.showConnectionMoveFeedback(request);
	}
	
	/**
	 * Erases connection move feedback. This method is called when a ReconnectRequest is
	 * received.
	 * @param request the reconnect request.
	 */
	protected void eraseConnectionMoveFeedback(ReconnectRequest request) {
		super.eraseConnectionMoveFeedback(request);
		if (originalSourceAnchor == null)
			return;
		getConnection().setSourceAnchor(originalSourceAnchor);
		originalSourceAnchor = null;
		getConnection().setTargetAnchor(originalTargetAnchor);
		originalTargetAnchor = null;
		/*
		if (originalAnchor == null)
			return;
		if (request.isMovingStartAnchor())
			getConnection().setSourceAnchor(originalAnchor);
		else
			getConnection().setTargetAnchor(originalAnchor);
		originalAnchor = null;
		feedbackHelper = null; */
	}
	
	private static class SeDiFeedbackHelper extends FeedbackHelper {
		public void update(ConnectionAnchor sourceAnchor, ConnectionAnchor targetAnchor) {
			if (sourceAnchor == null)
				throw new IllegalArgumentException("source anchor");
			//if (targetAnchor == null)
			//	throw new IllegalArgumentException("target anchor");
			getConnection().setSourceAnchor(sourceAnchor);
			getConnection().setTargetAnchor(targetAnchor);				
		}	
	}
}
