package no.uio.ifi.uml.sedi.figures;

import java.util.Enumeration;

import org.eclipse.draw2d.*;
import org.eclipse.draw2d.geometry.*;

public class CFFigure extends SeDiSubpartFigure {

	private Label op;

	public CFFigure() {
		setLayoutManager(new XYLayout());
		setBorder(new SeDiFrameBorder());
		//setBackgroundColor(ColorConstants.listBackground);
		//setOpaque(false);

		op = new Label("op");
		//op.setLocation(new Point(-23, 5));
		//op.setSize(new Dimension(90, 15));
		add(op);
		
		createAnchors();
		layoutAnchors(getSize().width, getSize().height);
		
		// Anchors need to be re-laid out when the figure is moved
		// in order to stay aligned with the "anchor steps"
		addFigureListener(new FigureListener() {
			public void figureMoved(IFigure source) {
				layoutAnchors(getSize().width, getSize().height);
			}
		});
		
		addAncestorListener(new AncestorListener.Stub() {
			private AnchorLayoutListener anchorLayoutListener;
			private FrameFigure parent;
			public void ancestorAdded(IFigure ancestor) {
				if (getParent() == ancestor) {
					anchorLayoutListener = new AnchorLayoutListener() {
						public void layoutChanged(int newOffset) {
							layoutAnchors(getSize().width, getSize().height);
						}
					};
					parent = ((FrameFigure)getParent()); 
					parent.getAnchorManager().
						addAnchorLayoutListener(anchorLayoutListener);					
				}
			}

			public void ancestorRemoved(IFigure ancestor) {
				if (ancestor == parent) {
					parent.getAnchorManager().
						removeAnchorLayoutListener(anchorLayoutListener);
					parent = null;
					anchorLayoutListener = null;
				}
			}
			
		});
	}

	public void layoutAnchors(final int width, final int height) {
		int startY = 25;
		if (getParent() != null) {
			AnchorManager anchorMgr = ((FrameFigure)getParent()).getAnchorManager();
			int ruler = startY + getBounds().y - anchorMgr.getVerticalAnchorOffset();
			startY += AnchorManager.ANCHOR_DELTA -
				ruler % AnchorManager.ANCHOR_DELTA;
		}
		
		int x = 0, y = startY, counter = 0;
		final Enumeration iter = anchors.elements();
		while (iter.hasMoreElements()) {
			if (counter == (anchors.size() / 2)) {
				x = width;
				y = startY;
			}
			SeDiAnchor anchor = (SeDiAnchor) iter.nextElement();
			anchor.setOffsetH(x);
			anchor.setOffsetV(y);
			
			if (y + AnchorManager.ANCHOR_DELTA < height)
				y += AnchorManager.ANCHOR_DELTA;
			
			counter++;
		}
	}

	// Called by CFCellEditorLocator to figure out where
	// to put the direct edit textbox.
	public Rectangle getTextBounds() {
		Rectangle rect = op.getBounds().getCopy();
		op.translateToAbsolute(rect);
		return rect;
	}
	
	// Called by the SeDiFrameBorder to paint a frame label border of the
	// correct size
	public Rectangle getLocalTextBounds() {
		return op.getBounds().getCopy();
	}		
	
	public Dimension getPreferredSize(int w, int h) {
		Dimension prefSize = super.getPreferredSize(w, h);
		Dimension defaultSize = new Dimension(200, 100);
		prefSize.union(defaultSize);
		return prefSize;
	}

	/*
	protected void paintFigure(Graphics g) {

		Rectangle r = getBounds().getCopy();
		int size = getText().length();

		// g.fillRectangle(r);
		g.drawLine(r.x, r.y, r.right() - 1, r.y);
		g.drawLine(r.x, r.bottom() - 1, r.right() - 1, r.bottom() - 1);
		g.drawLine(r.x, r.y, r.x, r.bottom() - 1);
		g.drawLine(r.right() - 1, r.bottom() - 1, r.right() - 1, r.y);

		g.drawLine(r.x, r.y + 20, r.x + 10 + 5 * size, r.y + 20);
		g
				.drawLine(r.x + 10 + 5 * size, r.y + 20, r.x + 25 + 5 * size,
						r.y + 11);
		g.drawLine(r.x + 25 + 5 * size, r.y + 11, r.x + 25 + 5 * size, r.y);

	}*/

	public String toString() {
		return "CFFigure";
	}

	public String getText() {
		return op.getText();
	}

	public void setText(String newText) {
		op.setText(newText);
		repaint();
	}

	public void validate() {
		if (isValid())
			return;
		super.validate();
		
		final Rectangle tr = op.getTextBounds();
		op.setSize(tr.width, tr.height);		
	}

	protected boolean useLocalCoordinates() {
		return true;
	}

	public boolean containsPoint(int x, int y) {
		Rectangle rect = getBounds().getCopy();
		int size = getText().length();
		// check if point is almost inside the pentagon
		if (x > rect.x && x < (rect.x + 20 + 5 * size)) {
			if (y > rect.y && y < rect.y + 20) {
				return true;
			}
		}

		// check if point is at the topline of the figure
		if (x > rect.x && x < rect.x + rect.width) {
			if (y > rect.y - 6 && y < rect.y + 6) {
				return true;
			}
		}

		// check if point is at the bottom line of the figure
		if (x > rect.x && x < rect.x + rect.width) {
			if (y > rect.y + rect.height - 6 && y < rect.y + rect.height + 6) {
				return true;
			}
		}

		// check if point is at the left line of the figure
		if (x > rect.x - 6 && x < rect.x + 6) {
			if (y > rect.y && y < rect.y + rect.height) {
				return true;
			}
		}

		// check if point is at the right line of the figure
		if (x > rect.x + rect.width - 6 && x < rect.x + rect.width + 6) {
			if (y > rect.y && y < rect.y + rect.height) {
				return true;
			}
		}

		return false;
	}

}
