package no.uio.ifi.uml.sedi.figures;

import java.util.Enumeration;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.XYLayout;
import org.eclipse.draw2d.text.FlowPage;
import org.eclipse.draw2d.text.TextFlow;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Insets;
import org.eclipse.draw2d.geometry.Rectangle;


public class FrameFigure extends SeDiSubpartFigure {

	private TextFlow text;
	private FlowPage titleFlow;
	private AnchorManager anchorManager = new AnchorManager();

	public FrameFigure() {
		setBorder(new SeDiFrameBorder());
		setLayoutManager(new XYLayout());
		setBackgroundColor(ColorConstants.listBackground);
		setOpaque(true);
		
		text = new TextFlow("");
		titleFlow = new FlowPage();
		titleFlow.add(new TextFlow("sd "));
		titleFlow.add(text);
		add(titleFlow);

		createAnchors();
		layoutAnchors(getSize().width, getSize().height);
		
		anchorManager.addAnchorLayoutListener(new AnchorLayoutListener() {
			public void layoutChanged(int newOffset) {
				layoutAnchors(getSize().width, getSize().height);
			}
		});
	}
	
	/*
	public void addNotify() {
		setFont(new Font(null, "Arial", 14, SWT.BOLD));		
		super.addNotify();
	}

	public void removeNotify() {
		super.removeNotify();

		// TODO This should always be done... how?
		Font font = getFont();
		setFont(null);
		font.dispose();
	} */

	// Called by FrameCellEditorLocator to figure out where
	// to put the direct edit textbox. Must return absolute coordinates. 
	public Rectangle getTextBounds() {
		Rectangle rect = text.getBounds().getCopy();
		text.translateToAbsolute(rect);
		return rect;
	}
	
	// Called by the SeDiFrameBorder to paint a frame label border of the
	// correct size
	public Rectangle getLocalTextBounds() {
		return text.getBounds().getCopy();
	}

	public Dimension getPreferredSize(int w, int h) {
		Dimension prefSize = super.getPreferredSize(w, h);
		Dimension defaultSize = new Dimension(200, 100);
		prefSize.union(defaultSize);
		return prefSize;
	}

	protected void paintFigure(Graphics graphics) {
		Rectangle rect = getBounds().getCopy();
		rect.crop(new Insets(2, 0, 2, 0));
		graphics.fillRectangle(rect);
	}

	public String toString() {
		return "FrameFigure";
	}

	public String getText() {
		return text.getText();
	}
	
	public AnchorManager getAnchorManager() {
		return anchorManager;
	}

	public void layoutAnchors(final int width, final int height) {
		
		// Calculate start y. This is a temporary solution until a better
		// approach is developed.
		int startY = 30;
		int ruler = startY - getAnchorManager().getVerticalAnchorOffset();
		startY += getInsets().top + AnchorManager.ANCHOR_DELTA -
			ruler % AnchorManager.ANCHOR_DELTA;
		
		int x = 0, y = startY, counter = 0;

		final Enumeration iter = anchors.elements();
		while (iter.hasMoreElements()) {
			if (counter == (anchors.size() / 2)) {
				x = width;
				y = startY;
			}
			SeDiAnchor anchor = (SeDiAnchor) iter.nextElement();
			anchor.setOffsetH(x);
			anchor.setOffsetV(y);
			
			if (y + AnchorManager.ANCHOR_DELTA < height)
				y += AnchorManager.ANCHOR_DELTA;
			
			counter++;
		}
	}

	public void setText(String newText) {
		text.setText(newText);
		repaint();
	}

	public void validate() {
		if (isValid())
			return;
		super.validate();

		Dimension prefSize = titleFlow.getPreferredSize();
		titleFlow.setSize(prefSize);
	}

	protected boolean useLocalCoordinates() {
		return true;
	}
}
