package no.uio.ifi.uml.sedi.figures;

import java.util.Enumeration;

import org.eclipse.draw2d.*;
import org.eclipse.draw2d.geometry.*;

public class IOFigure extends SeDiSubpartFigure {

	private IFigure pane;
	private Label ref;
	private Label text;

	public IOFigure() {
		setLayoutManager(new XYLayout());
		setBorder(new SeDiFrameBorder());
		setBackgroundColor(ColorConstants.listBackground);
		setOpaque(true);

		ref = new Label("ref");
		text = new Label();
		add(ref);
		add(text);

		createAnchors();
		layoutAnchors(getSize().width, getSize().height);

		// Anchors need to be re-laid out when the figure is moved
		// in order to stay aligned with the "anchor steps"
		addFigureListener(new FigureListener() {
			public void figureMoved(IFigure source) {
				layoutAnchors(getSize().width, getSize().height);
			}
		});
		
		addAncestorListener(new AncestorListener.Stub() {
			private AnchorLayoutListener anchorLayoutListener;
			private FrameFigure parent;
			public void ancestorAdded(IFigure ancestor) {
				if (getParent() == ancestor) {
					anchorLayoutListener = new AnchorLayoutListener() {
						public void layoutChanged(int newOffset) {
							layoutAnchors(getSize().width, getSize().height);
						}
					};
					parent = ((FrameFigure)getParent()); 
					parent.getAnchorManager().
						addAnchorLayoutListener(anchorLayoutListener);					
				}
			}

			public void ancestorRemoved(IFigure ancestor) {
				if (ancestor == parent) {
					parent.getAnchorManager().
						removeAnchorLayoutListener(anchorLayoutListener);
					parent = null;
					anchorLayoutListener = null;
				}
			}
			
		});		
	}
	
	public void layoutAnchors(final int width, final int height) {
		int startY = 25;
		if (getParent() != null) {
			AnchorManager anchorMgr = ((FrameFigure)getParent()).getAnchorManager();
			int ruler = startY + getBounds().y - anchorMgr.getVerticalAnchorOffset();
			startY += AnchorManager.ANCHOR_DELTA -
				ruler % AnchorManager.ANCHOR_DELTA;
		}
		
		int x = 0, y = startY, counter = 0;
		final Enumeration iter = anchors.elements();
		while (iter.hasMoreElements()) {
			if (counter == (anchors.size() / 2)) {
				x = width;
				y = startY;
			}
			SeDiAnchor anchor = (SeDiAnchor) iter.nextElement();
			anchor.setOffsetH(x);
			anchor.setOffsetV(y);
			
			if (y + AnchorManager.ANCHOR_DELTA < height)
				y += AnchorManager.ANCHOR_DELTA;
			
			counter++;
		}
	}

	public IFigure getContentsPane() {
		return pane;
	}

	// Called by IOCellEditorLocator to figure out where
	// to put the direct edit textbox.
	public Rectangle getTextBounds() {
		Rectangle rect = text.getBounds().getCopy();
		text.translateToAbsolute(rect);
		return rect;
	}

	// Called by the SeDiFrameBorder to paint a frame label 
	// border of the correct size
	public Rectangle getLocalTextBounds() {
		return ref.getBounds().getCopy();
	}	

	public Dimension getPreferredSize(int w, int h) {
		Dimension prefSize = super.getPreferredSize(w, h);
		Dimension defaultSize = new Dimension(200, 100);
		prefSize.union(defaultSize);
		return prefSize;
	}

	/*
	protected void paintFigure(Graphics g) {
		Rectangle r = getBounds().getCopy();
		g.fillRectangle(r);
		g.drawLine(r.x, r.y, r.right() - 1, r.y);
		g.drawLine(r.x, r.bottom() - 1, r.right() - 1, r.bottom() - 1);
		g.drawLine(r.x, r.y, r.x, r.bottom() - 1);
		g.drawLine(r.right() - 1, r.bottom() - 1, r.right() - 1, r.y);

		g.drawLine(r.x, r.y + 20, r.x + 22, r.y + 20);
		g.drawLine(r.x + 22, r.y + 20, r.x + 37, r.y + 11);
		g.drawLine(r.x + 37, r.y + 11, r.x + 37, r.y);
	}*/

	public String toString() {
		return "IOFigure"; //$NON-NLS-1$
	}

	public String getText() {
		return text.getText();
	}

	public void setText(String newText) {
		text.setText(newText);
	}

	public void validate() {
		if (isValid())
			return;
		super.validate();

		Rectangle tr = ref.getTextBounds();
		ref.setSize(tr.width, tr.height);

		final Rectangle r = getBounds();
		tr = text.getTextBounds();
		text.setLocation(
			new Point((r.width - tr.width) / 2, (r.height - tr.height) / 2));
		text.setSize(tr.width, tr.height);
	}

	protected boolean useLocalCoordinates() {
		return true;
	}
}
