
package no.uio.ifi.uml.sedi.uml2;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import no.uio.ifi.uml.sedi.SeDiEditor;
import no.uio.ifi.uml.sedi.SeDiPlugin;
import no.uio.ifi.uml.sedi.model.*;

import org.eclipse.core.resources.IFile;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.emf.common.util.EMap;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EAnnotation;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.uml2.Class;
import org.eclipse.uml2.Collaboration;
import org.eclipse.uml2.CombinedFragment;
import org.eclipse.uml2.EventOccurrence;
import org.eclipse.uml2.ExecutionOccurrence;
import org.eclipse.uml2.Gate;
import org.eclipse.uml2.Interaction;
import org.eclipse.uml2.InteractionOccurrence;
import org.eclipse.uml2.InteractionOperand;
import org.eclipse.uml2.InteractionOperator;
import org.eclipse.uml2.Lifeline;
import org.eclipse.uml2.Message;
import org.eclipse.uml2.MessageSort;
import org.eclipse.uml2.Model;
import org.eclipse.uml2.Operation;
import org.eclipse.uml2.Property;
import org.eclipse.uml2.Signal;
import org.eclipse.uml2.Stop;
import org.eclipse.uml2.Type;
import org.eclipse.uml2.UML2Factory;
import org.eclipse.uml2.UML2Package;
import org.eclipse.uml2.util.UML2Resource;



public class GenerateUML2 extends DebugUML2 {

	
	Model model;
	Collaboration collaboration;
	Interaction interaction;
	List classes = new ArrayList();
	List properties = new ArrayList();
	HashMap uml2Elements = new HashMap();
	List listOfLifelines=new ArrayList();
	List listOfCF = new ArrayList();
	List otherElements = new ArrayList();
	List listOfMessages = new ArrayList();
	List noReference = new ArrayList();
	HashMap referenceFixer = new HashMap();
	boolean valid=true;
	
	private Model createModel(String name) {
		Model model = UML2Factory.eINSTANCE.createModel();
		model.setName(name);

		return model;
	}

	public int generateUML2(SeDiEditor editor) {

		registerResourceFactories();

		final IFile file =((IFileEditorInput)editor.getEditorInput()).getFile();
		final String name = file.getName();
		final String ext = file.getFileExtension(); // extension w/o the "."
		
		final String modelName;
		if (ext == null)
			modelName = name;
		else
			modelName = name.substring(0, name.length() - (ext.length() + 1));
		
		model = createModel(modelName);
		collaboration = createCollaboration(model);

		/*
		 * Accessing the frame in the diagram
		 */
		Frame frame;
		List children=editor.getDiagram().getAllChildren();
		if (children.size() == 1 && (children.get(0) instanceof Frame))
			frame = (Frame) children.get(0);
		else
			return -1;
		
		interaction = createInteraction(collaboration,frame);			
		otherElements = findCombinedFragments(findLifelines(frame));
			
		createLifelines();	
		
		/*
		 * Adding messages to the otherElements list
		 */
		findMessages(frame);
		for(int i=0;i<listOfMessages.size();i++){
			otherElements.add(listOfMessages.get(i));
		}
		
		/*
		 * Adding combined fragments to the otherElements list
		 * and removing elements inside combined fragment from the
		 * otherElements list
		 */
		findCFElements();
		for(int i=0;i<listOfCF.size();i++){
			otherElements.add(listOfCF.get(i));
		}
		
		/*
		 * sorting the otherElements list so creation will
		 * be in the right order (from the top left and down)
		 */
		otherElements=sortList(otherElements,true);
		createElements(otherElements);
		
		fixRef();
		
		if(!valid){
			return -1;
			//TODO - get the dialog working
		}
		
		/* Finding the location and name of the source *.sd2 file
		 * Createing a *.uml2 file in the same loaction and with the same
		 * name
		 */
				
		if (SeDiPlugin.FILE_EXTENSION.equals(file.getFileExtension())) {	
			String filePath = file.getLocation().toOSString();
			if (ext != null)
				filePath = filePath.substring(0, filePath.length() - (ext.length() + 1));
			
			URI paths = URI.createFileURI(filePath);
			paths = paths.appendFileExtension(UML2Resource.FILE_EXTENSION);
			
			save(model, paths);					
		}
		else
			return -1;

		return 1;
	}

	private void fixRef() {
		Message m;
		Object part;
		for(int i=0;i<noReference.size();i++){
			m=(Message)noReference.get(i);
			part = referenceFixer.get(m);
			if(part instanceof CF){
				CombinedFragment uml2cf=(CombinedFragment)uml2Elements.get(part);
				Gate g = (Gate) uml2cf.createCfragmentGate(UML2Package.eINSTANCE.getGate());
				m.setReceiveEvent(g);
			}
			else if(part instanceof IO){
				InteractionOccurrence uml2io=(InteractionOccurrence)uml2Elements.get(part);
				Gate g = (Gate) uml2io.createActualGate(UML2Package.eINSTANCE.getGate());
				m.setReceiveEvent(g);
			}
		}
		
	}

	private void createElements(List elementList) {
		SeDiSubpart tmp;
				
		while(elementList.size()>0){
			tmp=(SeDiSubpart)elementList.get(0);
			if(tmp instanceof no.uio.ifi.uml.sedi.model.Message){
				createMessage((no.uio.ifi.uml.sedi.model.Message)tmp,interaction);
			}
			else if(tmp instanceof IO){
				createIO((IO)tmp,interaction);
			}
			else if(tmp instanceof CF){
				createCF((CF)tmp,interaction);
			}
			else if(tmp instanceof Continuation){
				createContinuation((Continuation)tmp,interaction);
			}
			else if(tmp instanceof Terminate){
				createTermination((Terminate)tmp,interaction);
			}
			else if(tmp instanceof Coregion){
				createCoregion((Coregion)tmp,interaction);
			}
			else if(tmp instanceof EO){
				createEO((EO)tmp,interaction);
			}
			elementList.remove(tmp);
		}
	}
	
	private void createEO(EO eo, Interaction interA) {
		ExecutionOccurrence uml2eo = (ExecutionOccurrence)interA.createFragment(UML2Package.eINSTANCE.getExecutionOccurrence());
		List coveredLifelines = findCoveredLifelines(eo);
		List cover=uml2eo.getCovereds();
		if(coveredLifelines.size()<1)
			return;
		for(int i=0;i<coveredLifelines.size();i++){
			cover.add(uml2Elements.get(coveredLifelines.get(i)));
		}
		EventOccurrence e1 = (EventOccurrence) interA.createFragment(UML2Package.eINSTANCE.getEventOccurrence());
		uml2eo.setStart(e1);
		cover=e1.getCovereds();
		cover.add(uml2Elements.get(coveredLifelines.get(0)));
		
		EventOccurrence e2 = (EventOccurrence) interA.createFragment(UML2Package.eINSTANCE.getEventOccurrence());
		uml2eo.setFinish(e2);
		cover=e2.getCovereds();
		cover.add(uml2Elements.get(coveredLifelines.get(0)));

		uml2Elements.put(eo,uml2eo);
	}

	private void createCoregion(Coregion coregion, Interaction interA) {
		// TODO - this is shorthand for a parallel combined fragment
		//		  each of the elements inside the coregion is an operand...
	}

	private void createTermination(Terminate terminate, Interaction interA) {
		
		Stop stop = (Stop)interA.createFragment(UML2Package.eINSTANCE.getStop());
		List coveredLifelines = findCoveredLifelines(terminate);
		List cover=stop.getCovereds();
		for(int i=0;i<coveredLifelines.size();i++){
			cover.add(uml2Elements.get(coveredLifelines.get(i)));
		}
		uml2Elements.put(terminate,stop);
	}

	private void createContinuation(Continuation continuation, Interaction interA) {
		org.eclipse.uml2.Continuation c = (org.eclipse.uml2.Continuation)interA.createFragment(UML2Package.eINSTANCE.getContinuation());
		List coveredLifelines = findCoveredLifelines(continuation);
		List cover=c.getCovereds();
		for(int i=0;i<coveredLifelines.size();i++){
			cover.add(uml2Elements.get(coveredLifelines.get(i)));
		}
		c.setName(continuation.getText());
		uml2Elements.put(continuation,c);
	}

	private void createCF(CF cf, Interaction interA) {
		CombinedFragment uml2cf = (CombinedFragment)interA.createFragment(UML2Package.eINSTANCE.getCombinedFragment());
		uml2Elements.put(cf,uml2cf);
		List coveredLifelines = findCoveredLifelines(cf);
		List covereds=uml2cf.getCovereds();
		for(int i=0;i<coveredLifelines.size();i++){
			covereds.add(uml2Elements.get(coveredLifelines.get(i)));
		}
		
		setRightOperator(uml2cf,cf.getText());
		
		createOperands(uml2cf,cf, coveredLifelines);
		
		
		/*
		 * TODO - find messages that crosses a cf and creates
		 * expression gates (cfragmentGates)
		 */
		
	}

	private void createOperands(CombinedFragment uml2cf, CF cf, List coveredLifelines) {
		List sep=findOperands(cf);
		InteractionOperand operand;
		
		List children=sortList(cf.getAllChildren(),true);
		Rectangle rect;
		for(int j=0;j<=sep.size();j++){
			operand = (InteractionOperand)uml2cf.createOperand(UML2Package.eINSTANCE.getInteractionOperand());
			
			rect = getCFRectangle(cf,j,sep);
			
			createCFElements(operand,cf,rect,children);
			
			List covereds=operand.getCovereds();
			for(int i=0;i<coveredLifelines.size();i++){
				covereds.add(uml2Elements.get(coveredLifelines.get(i)));
			}
		}
		cf.getAllChildren().clear();
	}

	private Rectangle getCFRectangle(CF cf, int j, List sep) {
		Rectangle r=new Rectangle();
		if(j==0){
			r.setLocation(cf.getLocation());
		}
		else r.setLocation(messageLocation((Separator)sep.get(j-1)));
		
		Dimension cfDim=cf.getSize();
		r.width=cfDim.width;
		
		int top,bottom;
		if(j==sep.size()){
			top=r.y;
			bottom=cf.getLocation().y+cf.getSize().height;
		}
		else {
			top=r.y;
			bottom=messageLocation((Separator)sep.get(j)).y;
		}
		r.height=bottom-top;
		return r;
	}

	private void createCFElements(InteractionOperand operand, CF cf, Rectangle r, List children) {
		
		SeDiSubpart part;
	
		for(int i=0;i<children.size();i++){
			part = (SeDiSubpart)children.get(i);
			if(insideOperand(part,r)){
				if(part instanceof no.uio.ifi.uml.sedi.model.Message){
					createMessage((no.uio.ifi.uml.sedi.model.Message)part,operand);
				}
				else if(part instanceof IO){
					createIO((IO)part,operand);
				}
				else if(part instanceof CF){
					createCF((CF)part,operand);
				}
				else if(part instanceof EO){
					createEO((EO)part,operand);
				}
				else if(part instanceof Terminate){
					createTerminate((Terminate)part,operand);
				}
				else if(part instanceof Continuation){
					createContinuation((Continuation)part,operand);
				}
			}
		}
		
	}

	private void createContinuation(Continuation continuation, InteractionOperand operand) {
		org.eclipse.uml2.Continuation c = (org.eclipse.uml2.Continuation)operand.createFragment(UML2Package.eINSTANCE.getContinuation());
		List coveredLifelines = findCoveredLifelines(continuation);
		List cover=c.getCovereds();
		for(int i=0;i<coveredLifelines.size();i++){
			cover.add(uml2Elements.get(coveredLifelines.get(i)));
		}
		c.setName(continuation.getText());
		uml2Elements.put(continuation,c);
		
	}

	private void createTerminate(Terminate terminate, InteractionOperand operand) {
		Stop stop = (Stop)operand.createFragment(UML2Package.eINSTANCE.getStop());
		List coveredLifelines = findCoveredLifelines(terminate);
		List cover=stop.getCovereds();
		for(int i=0;i<coveredLifelines.size();i++){
			cover.add(uml2Elements.get(coveredLifelines.get(i)));
		}
		uml2Elements.put(terminate,stop);
		
	}

	private void createEO(EO eo, InteractionOperand operand) {
		ExecutionOccurrence uml2eo = (ExecutionOccurrence)operand.createFragment(UML2Package.eINSTANCE.getExecutionOccurrence());
		List coveredLifelines = findCoveredLifelines(eo);
		List cover=uml2eo.getCovereds();
		if(coveredLifelines.size()<1)
			return;
		for(int i=0;i<coveredLifelines.size();i++){
			cover.add(uml2Elements.get(coveredLifelines.get(i)));
		}
		EventOccurrence e1 = (EventOccurrence) operand.createFragment(UML2Package.eINSTANCE.getEventOccurrence());
		uml2eo.setStart(e1);
		cover=e1.getCovereds();
		cover.add(uml2Elements.get(coveredLifelines.get(0)));
		
		EventOccurrence e2 = (EventOccurrence) operand.createFragment(UML2Package.eINSTANCE.getEventOccurrence());
		uml2eo.setFinish(e2);
		cover=e2.getCovereds();
		cover.add(uml2Elements.get(coveredLifelines.get(0)));

		uml2Elements.put(eo,uml2eo);
	}

	private void createCF(CF cf, InteractionOperand operand) {
		CombinedFragment uml2cf = (CombinedFragment)operand.createFragment(UML2Package.eINSTANCE.getCombinedFragment());
		uml2Elements.put(cf,uml2cf);
		List coveredLifelines = findCoveredLifelines(cf);
		List covereds=uml2cf.getCovereds();
		for(int i=0;i<coveredLifelines.size();i++){
			covereds.add(uml2Elements.get(coveredLifelines.get(i)));
		}
		
		setRightOperator(uml2cf,cf.getText());
		
		createOperands(uml2cf,cf, coveredLifelines);
	}

	private void createIO(IO io, InteractionOperand operand) {
		InteractionOccurrence interO = (InteractionOccurrence) operand.createFragment(UML2Package.eINSTANCE.getInteractionOccurrence());
		List coverLifelines = findCoveredLifelines(io);
		
		List covereds=interO.getCovereds();
		for(int i=0;i<coverLifelines.size();i++){
			covereds.add(uml2Elements.get(coverLifelines.get(i)));
		}
		interO.setName(io.getText());
		uml2Elements.put(io,interO);
		
	}

	private void createMessage(no.uio.ifi.uml.sedi.model.Message currMessage, InteractionOperand operand) {
		
//		Creating message
		Message m = (Message) interaction.createMessage(UML2Package.eINSTANCE.getMessage());
		m.setName(currMessage.getText());
		//Setting the sort of the message
		if(currMessage instanceof AMessage){
			m.setMessageSort(MessageSort.ASYNCH_CALL_LITERAL);
			Object check = getOwnedMember(model, m.getName());
			//Object check=model.getOwnedMember(m.getName());
			if(check==null){
				Signal s=(Signal) model.createOwnedMember(UML2Package.eINSTANCE.getSignal());
				s.setName(m.getName());
				m.setSignature(s);
			}
		}
		else if(currMessage instanceof CMessage)
			m.setMessageSort(MessageSort.SYNCH_CALL_LITERAL);
		else if(currMessage instanceof RMessage){
			m.setMessageSort(MessageSort.SYNCH_SIGNAL_LITERAL);
			EAnnotation ea=m.createEAnnotation("keywords");
			EMap details = ea.getDetails();
			details.put("return",null);
		}
		SeDiSubpart source = currMessage.getSource();
		if(source instanceof no.uio.ifi.uml.sedi.model.Lifeline){
			EventOccurrence e1 = (EventOccurrence) operand.createFragment(UML2Package.eINSTANCE.getEventOccurrence());
			m.setSendEvent(e1);
			Lifeline uml2l1 = (Lifeline)uml2Elements.get(source);
			List coveredBy1 = uml2l1.getCoveredBys();
			coveredBy1.add(e1);
			if(currMessage instanceof RMessage){
				// finding the class the lifeline represents and adding an operation
					Property p = (Property)uml2l1.getRepresents();
					Class c = (Class)p.getType();
					Object check=c.getOwnedOperation(currMessage.getText());
					if(check==null){
						Operation o = c.createOwnedOperation(UML2Package.eINSTANCE.getOperation());
						o.setName(currMessage.getText());
						m.setSignature(o);
					}
				}
		}
		else if(source instanceof IO){
			InteractionOccurrence uml2io = (InteractionOccurrence) uml2Elements.get(source);
			Gate g = (Gate) uml2io.createActualGate(UML2Package.eINSTANCE.getGate());
			m.setSendEvent(g);
		}
		else if(source instanceof CF){
			CombinedFragment uml2cf = (CombinedFragment) uml2Elements.get(source);
			Gate g = (Gate) uml2cf.createCfragmentGate(UML2Package.eINSTANCE.getGate());
			m.setSendEvent(g);
		}
		else if(source instanceof Frame){
			Gate g = (Gate)interaction.createFormalGate(UML2Package.eINSTANCE.getGate());
			m.setSendEvent(g);
		}
		SeDiSubpart target = currMessage.getTarget();
		if(target instanceof no.uio.ifi.uml.sedi.model.Lifeline){
				EventOccurrence e2 = (EventOccurrence) operand.createFragment(UML2Package.eINSTANCE.getEventOccurrence());
				m.setReceiveEvent(e2);
				Lifeline uml2l2 = (Lifeline)uml2Elements.get(target);
				List coveredBy2=uml2l2.getCoveredBys();
				coveredBy2.add(e2);
				if(currMessage instanceof CMessage){
					// finding the class the lifeline represents and adding a operation
					Property p = (Property)uml2l2.getRepresents();
					Class c = (Class)p.getType();
					Object check=c.getOwnedOperation(currMessage.getText());
					if(check==null){
						Operation o = c.createOwnedOperation(UML2Package.eINSTANCE.getOperation());
						o.setName(currMessage.getText());
						m.setSignature(o);
					}
				}
		}
		else if(target instanceof IO){
			InteractionOccurrence uml2io = (InteractionOccurrence) uml2Elements.get(target);
			if(uml2io==null){
				noReference.add(m);
				referenceFixer.put(m,target);
				uml2Elements.put(currMessage,m);
				return;
			}
			Gate g = (Gate) uml2io.createActualGate(UML2Package.eINSTANCE.getGate());
			m.setReceiveEvent(g);
		}
		else if(target instanceof CF){
			CombinedFragment uml2cf = (CombinedFragment) uml2Elements.get(target);
			if(uml2cf==null){
				noReference.add(m);
				referenceFixer.put(m,target);
				uml2Elements.put(currMessage,m);
				return;
			}
			Gate g = (Gate) uml2cf.createCfragmentGate(UML2Package.eINSTANCE.getGate());
			m.setReceiveEvent(g);
		}
		else if(target instanceof Frame){
			Gate g = (Gate) interaction.createFormalGate(UML2Package.eINSTANCE.getGate());
			m.setReceiveEvent(g);
		}
		uml2Elements.put(currMessage,m);
	}

	private boolean insideOperand(SeDiSubpart part, Rectangle r) {
		Point p;
		if(part instanceof Separator){
			return false;
		}
		if(part instanceof no.uio.ifi.uml.sedi.model.Message){
			p = messageLocation((no.uio.ifi.uml.sedi.model.Message)part);
		}
		else p=part.getLocation();
		
		if(p.y<r.y)
			return false;
		if(p.y>r.y+r.height)
			return false;
		
		return true;
	}

	private List findOperands(CF cf) {
		List sep=new ArrayList();
		Vector separators=cf.getSourceConnections();
		
		for(Enumeration e = separators.elements();e.hasMoreElements();){
			SeDiSubpart part = (SeDiSubpart)e.nextElement();
			if(part instanceof Separator){
				sep.add(part);
			}
		}
		return sep;
	}

	private void setRightOperator(CombinedFragment combinedFragment, String text) {
		if(text.equalsIgnoreCase("seq")){
			combinedFragment.setInteractionOperator(InteractionOperator.SEQ_LITERAL);
		}
		else if(text.equalsIgnoreCase("alt")){
			combinedFragment.setInteractionOperator(InteractionOperator.ALT_LITERAL);
		}
			
		else if(text.equalsIgnoreCase("opt")){
			combinedFragment.setInteractionOperator(InteractionOperator.OPT_LITERAL);
		}
			
		else if(text.equalsIgnoreCase("break")){
			combinedFragment.setInteractionOperator(InteractionOperator.BREAK_LITERAL);
		}
			
		else if(text.equalsIgnoreCase("par")){
			combinedFragment.setInteractionOperator(InteractionOperator.PAR_LITERAL);
		}
			
		else if(text.equalsIgnoreCase("strict")){
			combinedFragment.setInteractionOperator(InteractionOperator.STRICT_LITERAL);
		}
			
		else if(text.equalsIgnoreCase("loop")){
			combinedFragment.setInteractionOperator(InteractionOperator.LOOP_LITERAL);
		}
			
		else if(text.equalsIgnoreCase("critical")){
			combinedFragment.setInteractionOperator(InteractionOperator.CRITICAL_LITERAL);
		}
			
		else if(text.equalsIgnoreCase("neg")){
			combinedFragment.setInteractionOperator(InteractionOperator.NEG_LITERAL);
		}
			
		else if(text.equalsIgnoreCase("assert")){
			combinedFragment.setInteractionOperator(InteractionOperator.ASSERT_LITERAL);
		}
			
		else if(text.equalsIgnoreCase("ignore")){
			combinedFragment.setInteractionOperator(InteractionOperator.IGNORE_LITERAL);
		}
			
		else if(text.equalsIgnoreCase("consider")){
			combinedFragment.setInteractionOperator(InteractionOperator.CONSIDER_LITERAL);
		}
			
		
	}

	private void createMessage(no.uio.ifi.uml.sedi.model.Message currMessage,Interaction interA) {
		
		//Creating message
		Message m = (Message) interA.createMessage(UML2Package.eINSTANCE.getMessage());
		m.setName(currMessage.getText());
		//Setting the sort of the message
		if(currMessage instanceof AMessage){
			m.setMessageSort(MessageSort.ASYNCH_CALL_LITERAL);
			Object check = getOwnedMember(model, m.getName());
			//Object check=model.getOwnedMember(m.getName());
			if(check==null){
				Signal s=(Signal) model.createOwnedMember(UML2Package.eINSTANCE.getSignal());
				s.setName(m.getName());
				m.setSignature(s);
			}
		}
		else if(currMessage instanceof CMessage)
			m.setMessageSort(MessageSort.SYNCH_CALL_LITERAL);
		else if(currMessage instanceof RMessage){
			m.setMessageSort(MessageSort.SYNCH_SIGNAL_LITERAL);
			EAnnotation ea=m.createEAnnotation("keywords");
			EMap details = ea.getDetails();
			details.put("return",null);
		}
		SeDiSubpart source = currMessage.getSource();
		if(source instanceof no.uio.ifi.uml.sedi.model.Lifeline){
			EventOccurrence e1 = (EventOccurrence) interA.createFragment(UML2Package.eINSTANCE.getEventOccurrence());
			m.setSendEvent(e1);
			Lifeline uml2l1 = (Lifeline)uml2Elements.get(source);
			List coveredBy1 = uml2l1.getCoveredBys();
			coveredBy1.add(e1);
			if(currMessage instanceof RMessage){
				//	 finding the class the lifeline represents and adding an operation
				Property p = (Property)uml2l1.getRepresents();
				Class c = (Class)p.getType();
				Object check=c.getOwnedOperation(currMessage.getText());
				if(check==null){
					Operation o = c.createOwnedOperation(UML2Package.eINSTANCE.getOperation());
					o.setName(currMessage.getText());
					m.setSignature(o);
				}
			}
		}
		else if(source instanceof IO){
			InteractionOccurrence uml2io = (InteractionOccurrence) uml2Elements.get(source);
			Gate g = (Gate) uml2io.createActualGate(UML2Package.eINSTANCE.getGate());
			m.setSendEvent(g);
		}
		else if(source instanceof CF){
			CombinedFragment uml2cf = (CombinedFragment) uml2Elements.get(source);
			Gate g = (Gate) uml2cf.createCfragmentGate(UML2Package.eINSTANCE.getGate());
			m.setSendEvent(g);
		}
		else if(source instanceof Frame){
			Gate g = (Gate)interaction.createFormalGate(UML2Package.eINSTANCE.getGate());
			m.setSendEvent(g);
		}
		SeDiSubpart target = currMessage.getTarget();
		if(target instanceof no.uio.ifi.uml.sedi.model.Lifeline){
				EventOccurrence e2 = (EventOccurrence) interA.createFragment(UML2Package.eINSTANCE.getEventOccurrence());
				m.setReceiveEvent(e2);
				Lifeline uml2l2 = (Lifeline)uml2Elements.get(target);
				List coveredBy2=uml2l2.getCoveredBys();
				coveredBy2.add(e2);
		
				if(currMessage instanceof CMessage){
				// finding the class the lifeline represents and adding an operation
					Property p = (Property)uml2l2.getRepresents();
					Class c = (Class)p.getType();
					Object check=c.getOwnedOperation(currMessage.getText());
					if(check==null){
						Operation o = c.createOwnedOperation(UML2Package.eINSTANCE.getOperation());
						o.setName(currMessage.getText());
						m.setSignature(o);
					}
				}
		}
		else if(target instanceof IO){
			InteractionOccurrence uml2io = (InteractionOccurrence) uml2Elements.get(target);
			if(uml2io==null){
				noReference.add(m);
				referenceFixer.put(m,target);
				uml2Elements.put(currMessage,m);
				return;
			}
			Gate g = (Gate) uml2io.createActualGate(UML2Package.eINSTANCE.getGate());
			m.setReceiveEvent(g);
		}
		else if(target instanceof CF){
			CombinedFragment uml2cf = (CombinedFragment) uml2Elements.get(target);
			if(uml2cf==null){
				noReference.add(m);
				referenceFixer.put(m,target);
				uml2Elements.put(currMessage,m);
				return;
			}
			Gate g = (Gate) uml2cf.createCfragmentGate(UML2Package.eINSTANCE.getGate());
			m.setReceiveEvent(g);
		}
		else if(target instanceof Frame){
			Gate g = (Gate) interaction.createFormalGate(UML2Package.eINSTANCE.getGate());
			m.setReceiveEvent(g);
		}
		uml2Elements.put(currMessage,m);
	}

	private void createIO(IO io, Interaction interA) {
		InteractionOccurrence interO = (InteractionOccurrence) interA.createFragment(UML2Package.eINSTANCE.getInteractionOccurrence());
		List coverLifelines = findCoveredLifelines(io);
		
		List covereds=interO.getCovereds();
		for(int i=0;i<coverLifelines.size();i++){
			covereds.add(uml2Elements.get(coverLifelines.get(i)));
		}
		interO.setName(io.getText());
		uml2Elements.put(io,interO);
	}

	private void createLifelines() {
		no.uio.ifi.uml.sedi.model.Lifeline currLifeline;
		Class currClass;
		Property currProperty;
		org.eclipse.uml2.Lifeline currUML2Lifeline;
		for(int i=0;i<listOfLifelines.size();i++){
			currLifeline = (no.uio.ifi.uml.sedi.model.Lifeline) listOfLifelines.get(i);
			currClass = createClass(model,currLifeline);
			classes.add(currClass);
			currProperty=createProperty(collaboration,currClass,currLifeline);
			properties.add(currProperty);
			currUML2Lifeline=createLifeline(interaction,currProperty);
			uml2Elements.put(currLifeline,currUML2Lifeline);
		}
		
	}

	private  org.eclipse.uml2.Lifeline createLifeline(Interaction interaction, Property currProperty) {
		org.eclipse.uml2.Lifeline l = (org.eclipse.uml2.Lifeline)interaction.createLifeline(UML2Package.eINSTANCE.getLifeline());
		l.setName(currProperty.getName());
		l.setRepresents(currProperty);
		return l;
	}

	private Collaboration createCollaboration(Model model) {
		Collaboration c=(Collaboration)model.createOwnedMember(UML2Package.eINSTANCE.getCollaboration());
		c.setName("Null");
	
		return c;
	}

	private Property createProperty(Collaboration collaboration, Class curr, no.uio.ifi.uml.sedi.model.Lifeline currLifeline) {
		Property p;
		Object check;
		
		int index=currLifeline.getText().indexOf(":");
		if(index>-1){
			check=collaboration.getAttribute(currLifeline.getText().substring(0,index));
			if(check!=null){
				if(check instanceof Property){
					Class c=(Class) ((Property)check).getType();
					if(c.equals(curr))
						return (Property) check;
				}
			}
			p = (Property)collaboration.createOwnedAttribute(UML2Package.eINSTANCE.getProperty());
			p.setName(currLifeline.getText().substring(0,index));
		}
		else {
			check=collaboration.getAttribute(curr.getName());
			if(check!=null){
				if(check instanceof Property){
					Class c=(Class) ((Property)check).getType();
					if(c.equals(curr))
						return (Property)check;
				}				
			}
			p = (Property)collaboration.createOwnedAttribute(UML2Package.eINSTANCE.getProperty());
			p.setName(curr.getName());
		}
		p.setType((Type)curr);
		return p;
	}

	private org.eclipse.uml2.Class createClass(Model model, no.uio.ifi.uml.sedi.model.Lifeline l) {
		
		Object check;
		Class c;
		int index=l.getText().indexOf(":");
		
		if(index>-1&&l.getText().length()>1){
			check = getOwnedMember(model, l.getText().substring(index+1));
			//check=model.getOwnedMember(l.getText().substring(index+1));
			if(check!=null){
				if(check instanceof Class){
					return (Class)check;
				}
			}
			c=(Class)model.createOwnedMember(UML2Package.eINSTANCE.getClass_());
			c.setName(l.getText().substring(index+1));
		}
		else {
			check = getOwnedMember(model, l.getText());
			//check=model.getOwnedMember(l.getText());
			if(check!=null){
				if(check instanceof Class){
					return (Class)check;
				}
			}
			c=(Class)model.createOwnedMember(UML2Package.eINSTANCE.getClass_());
			c.setName(l.getText());
		}
		return c;
	}

	private Interaction createInteraction(Collaboration c, Frame frame) {
		Interaction i = (Interaction)c.createOwnedBehavior(UML2Package.eINSTANCE.getInteraction());
		i.setName(frame.getText());
		return i;
	}

	/*
	 * Nesting CF
	 */
	private void addCFToCF() {
		List tmpList=new ArrayList();
		Object tmp;
		
		for(int i=0;i<listOfCF.size();i++){
			tmp=listOfCF.get(i);
			for(int j=0;j<listOfCF.size();j++){
				if(j!=i&&inside((CF) tmp,(CF) listOfCF.get(j))){
					((CF)listOfCF.get(j)).add((CF)tmp);
					tmpList.add(tmp);
					break;
				}
			}
		}
		for(int i=0;i<tmpList.size();i++){
			listOfCF.remove(tmpList.get(i));
		}
	}

	private void findCFElements() {
		
		
		listOfCF=sortList(listOfCF,true);
		Object tmp;
		
		/*
		 * Sorting the cf list so the addOtherElementsToCF works
		 */
		for(int j=0;j<listOfCF.size();j++)
			for(int i=j;i>0 && inside((CF)listOfCF.get(i),(CF)listOfCF.get(i-1));i--){
				tmp=listOfCF.get(i);
				listOfCF.set(i,listOfCF.get(i-1));
				listOfCF.set(i-1,tmp);
		 }
		
		addOtherElementsToCF();
		addCFToCF();
		
	}

	private void addOtherElementsToCF() {
	
		List tmpList = new ArrayList();
		Iterator it = otherElements.iterator();
		SeDiSubpart tmp;
		
		while(it.hasNext()){
			tmp=(SeDiSubpart)it.next();
			for(int i=0;i<listOfCF.size();i++){
				if(insideCF(tmp,(CF) listOfCF.get(i))){
					((CF)listOfCF.get(i)).add(tmp);
					tmpList.add(tmp);
					break;
				}
			}
		}
		
		for(int i=0;i<tmpList.size();i++){
			otherElements.remove(tmpList.get(i));
			
		}
	}

	private boolean insideCF(SeDiSubpart part, CF cf) {
		/*
		 * check if the part is inside the CF
		 */
		int partx,party,cfx,cfy;
		if(part instanceof no.uio.ifi.uml.sedi.model.Message){
			Point p = messageLocation((no.uio.ifi.uml.sedi.model.Message) part);
			partx=p.x;
			party=p.y;
		}
		else{
			partx=part.getLocation().x;
			party=part.getLocation().y;
		}
		cfx=cf.getLocation().x;
		cfy=cf.getLocation().y;
		
		if(partx<cfx)
			return false;
		if(party<cfy)
			return false;
		if(partx>=cfx+cf.getSize().width)
			return false;
		if(party>cfy+cf.getSize().height)
			return false;
		return true;
	}

	private boolean inside(CF cf2, CF cf1) {
		
		int cf1x,cf2x,cf1y,cf2y;
		
		cf1x=cf1.getLocation().x;
		cf2x=cf2.getLocation().x;
		cf1y=cf1.getLocation().y;
		cf2y=cf2.getLocation().y;
		
		if(cf2x<cf1x)
			return false;
		
		if(cf2y<cf1y)
			return false;
		
		if((cf2x+cf2.getSize().width)>(cf1x+cf1.getSize().width))
			return false;
		
		if((cf2y+cf2.getSize().height)>(cf1y+cf1.getSize().height))
			return false;
		
		return true;
	}

	private List findCoveredLifelines(SeDiSubpart part) {
		List coveredBy = new ArrayList();
		no.uio.ifi.uml.sedi.model.Lifeline tmp;
		int xcoord,partxcoord;
		for(int i=0;i<listOfLifelines.size();i++){
			tmp = (no.uio.ifi.uml.sedi.model.Lifeline)listOfLifelines.get(i);
			xcoord = tmp.getLocation().x+tmp.getSize().width/2;
			partxcoord = part.getLocation().x;
			if(xcoord>partxcoord && xcoord<(part.getSize().width+partxcoord))
				coveredBy.add(tmp);
		}
			
		return coveredBy;
	}

	private void findMessages(Frame frame) {
		
		no.uio.ifi.uml.sedi.model.Lifeline currLifeline;
		Vector currMessages;
		
		for(int i=0;i<listOfLifelines.size();i++){
			currLifeline = (no.uio.ifi.uml.sedi.model.Lifeline) listOfLifelines.get(i);
			currMessages=currLifeline.getSourceConnections();
			for(Enumeration e = currMessages.elements();e.hasMoreElements();){
				listOfMessages.add((no.uio.ifi.uml.sedi.model.Message)e.nextElement());
			}
		}
		for(int i=0;i<otherElements.size();i++){
			SeDiSubpart part = (SeDiSubpart) otherElements.get(i);
				if(part instanceof IO){
					currMessages=part.getSourceConnections();
					for(Enumeration e = currMessages.elements();e.hasMoreElements();){
						listOfMessages.add((no.uio.ifi.uml.sedi.model.Message)e.nextElement());
					}
				}
				
		}
		for(int i=0;i<listOfCF.size();i++){
			CF part= (CF) listOfCF.get(i);
			currMessages=part.getSourceConnections();
			for(Enumeration e = currMessages.elements();e.hasMoreElements();){
				listOfMessages.add((no.uio.ifi.uml.sedi.model.Message)e.nextElement());
			}
		}
		currMessages=frame.getSourceConnections();
		for(Enumeration e = currMessages.elements();e.hasMoreElements();){
			listOfMessages.add((no.uio.ifi.uml.sedi.model.Message)e.nextElement());
		}
	}

	

	private Point messageLocation(no.uio.ifi.uml.sedi.model.Message m){
		
		String anchorName = m.getSourceRef();
		Point p = new Point();
		int xcoord,ycoord;
		/*
		 * Lifeline.y=40 + 44(head) +13(space between anchors)*(anchorName-1)
		 * 
		 */
		SeDiSubpart part = m.getSource();
		
		 
		
		if(part instanceof no.uio.ifi.uml.sedi.model.Lifeline){
		
			no.uio.ifi.uml.sedi.model.Lifeline ll = (no.uio.ifi.uml.sedi.model.Lifeline)part;
			xcoord=ll.getLocation().x+ll.getSize().width/2;
			ycoord=((new Integer(anchorName)).intValue()-1)*13+84;
			p.setLocation(xcoord,ycoord);
			}

		
		else if(part instanceof IO){
			IO io=(IO)part;
			int anchor = (new Integer(anchorName)).intValue();
			if(anchor<51){
				xcoord=io.getLocation().x;
				ycoord=io.getLocation().y+30+((anchor-1)*13);
			}
			else{
				xcoord=io.getLocation().x+io.getSize().width;
				ycoord=io.getLocation().y+30+((anchor-20)*13);
			}
			p.setLocation(xcoord,ycoord);
		}
		else if(part instanceof CF){
			CF cf=(CF)part;
			int anchor = (new Integer(anchorName)).intValue();
			if(anchor<51){
				xcoord=cf.getLocation().x;
				ycoord=cf.getLocation().y+30+((anchor-1)*13);
			}
			else{
				xcoord=cf.getLocation().x+cf.getSize().width;
				ycoord=cf.getLocation().y+30+((anchor-20)*13);
			}
			p.setLocation(xcoord,ycoord);
		}
		else if(part instanceof Frame){
			Frame frame = (Frame)part;
			int anchor = (new Integer(anchorName)).intValue();
			if(anchor<51){
				xcoord=frame.getLocation().x;
				ycoord=frame.getLocation().y+30+((anchor-1)*26);
			}
			else{
				xcoord=frame.getLocation().x+frame.getSize().width;
				ycoord=frame.getLocation().y+30+((anchor-20)*26);
			}
			p.setLocation(xcoord,ycoord);
		}
		return p;
	}
	
	private List findCombinedFragments(List tmp) {
		List elements = new ArrayList();
		for(int i=0;i<tmp.size();i++){
			if(tmp.get(i) instanceof CF){
				listOfCF.add(tmp.get(i));
			}
			else elements.add(tmp.get(i));
		}
		return elements;
	}

	private List findLifelines(Frame frame){
		
		List elements = new ArrayList();
		List children = frame.getAllChildren();
		
		SeDiSubpart tmp;
		
		for(int i=0;i<children.size();i++){
		
			tmp=(SeDiSubpart)children.get(i);
			if(tmp instanceof no.uio.ifi.uml.sedi.model.Lifeline){
				listOfLifelines.add(tmp);
			}
			else{
				elements.add(tmp);
			}
			
		}
		listOfLifelines=sortList(listOfLifelines,false);
		return elements;
	}


	private List sortList(List unsortedList, boolean ycoord) {
		
		List sortedList = new ArrayList();
		no.uio.ifi.uml.sedi.model.SeDiSubpart tmp;
		
		while(unsortedList.size()>0){
			tmp = removeMin(unsortedList,ycoord);
			sortedList.add(tmp);
		}
		return sortedList;
	}

	private no.uio.ifi.uml.sedi.model.SeDiSubpart removeMin(List tmpList, boolean ycoord) {
		
		int min=Integer.MAX_VALUE;
		int coord;
		SeDiSubpart tmp;
		SeDiSubpart subpart = null;
		
		for(int i=0;i<tmpList.size();i++){
			tmp=(SeDiSubpart)tmpList.get(i);
			if(ycoord){
				if(tmp instanceof no.uio.ifi.uml.sedi.model.Message)
					coord=messageLocation((no.uio.ifi.uml.sedi.model.Message)tmp).y;
				else
					coord=tmp.getLocation().y;
			}
			else{
				if(tmp instanceof no.uio.ifi.uml.sedi.model.Message)
					coord=messageLocation((no.uio.ifi.uml.sedi.model.Message)tmp).x;
				else
					coord=tmp.getLocation().x;
			}
			if(coord<min){
				min=coord;
				subpart=tmp;
			}
				
		}
		tmpList.remove(subpart);
		return subpart;
	}

	private void save(org.eclipse.uml2.Package p, URI uri) {
		Resource resource = new ResourceSetImpl().createResource(uri);
		resource.getContents().add(p);

		try {
			resource.save(null);
			//out("UML2 file created!");
		} catch (IOException ioe) {
			err(ioe.getMessage());
		}
	}
	protected static void registerResourceFactories() {
		Resource.Factory.Registry.INSTANCE.getExtensionToFactoryMap().put(
			UML2Resource.FILE_EXTENSION, UML2Resource.Factory.INSTANCE);
	}
	
	// The API has been changed between UML2 1.0.x and 1.1.x, meaning that the
	// method "getOwnedMember" has a different signature. In order to be
	// runnable on both versions of the UML2 API, we need to catch the error
	// that occurs on the wrong API version and try an alternate approach
	private Object getOwnedMember(Model model, String name) {
		try {
			// 3.1
			return model.getOwnedMember(name);
		}
		catch (NoSuchMethodError e) {
			// 3.0 The only difference is the return type
			//     The changes are compile time compatible but not runtime compatible
			try {
				java.lang.Class modelClass = model.getClass();
				java.lang.reflect.Method m = 
					modelClass.getMethod("getOwnedMember", 
							             new java.lang.Class[] { String.class });
				return m.invoke(model, new Object[] { name });
			}
			catch (Exception nsme) {
				throw new IncompatibleClassChangeError("Can't call UML2 getOwnedMember: " + e.getMessage());
			}
		}
	}
}